#
# Copyright (c) 1999-2002 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

# 
# All of the architecture dependent makefile magic is done here.
# This fragment is included by most of the subdirectories.
#

#
# Options:
#	NODEOS_SPEC_COMPAT	include defines for StudlyCaps API names
#
# OSKit Options:
#	MEMDEBUG		include memdebug support (for debugging)
#	MEMLOGGING		log memory alloc/frees and dump unmatched
#				events at exit time
#	OSKIT_STRIDE_SCHED	use the CPU stride scheduler
#				(requires the OSKit be build with stride sched)
#	USE_POLLING		(incomplete) use a polling model to access
#				channel devices
#

# flags and libs common to oskit and oskit/unix.
OSKIT_ARCH_CFLAGS=-pthread -posix-oskit #-DMEMLOGGING #-DNODEOS_SPEC_COMPAT
OSKIT_ARCH_LIBS=-loskit_startup -loskit_linux_dev -loskit_bootp \
		-loskit_freebsd_net -loskit_fsnamespace

#
# Enable testbed hooks when running on emulab.net (OSKit only)
#
#OSKIT_ARCH_CFLAGS += -DUTAHTESTBED
#OSKIT_ARCH_LIBS += -loskit_tmcp -loskit_bootp

#
# Enable CPU proportional share scheduling (OSKit only)
# (OSKit pthread library must be configured for stride scheduling)
#
#OSKIT_ARCH_CFLAGS += -DOSKIT_STRIDE_SCHED

#
# Enable polled device interface
# (OSKit linuxdev library must be configured for device polling)
#
#OSKIT_ARCH_CFLAGS += -DUSE_POLLING

# uncomment to use memdebug library (OSKit/OSKitunix only)
OSKIT_ARCH_CFLAGS += -DMEMDEBUG
OSKIT_ARCH_LIBS += -loskit_memdebug

ifeq ($(HOST_OS),oskit)
ARCH_CFLAGS =   $(OSKIT_ARCH_CFLAGS) 
ARCH_LIBS =	$(OSKIT_ARCH_LIBS) -loskit_netbsd_fs -loskit_diskpart \
		-loskit_memfs
ARCH_PLIBS =	$(addsuffix _p, $(ARCH_LIBS)) -loskit_memfs_p -loskit_gprof
ARCH_DEPINCS =	-I$(OSKIT_INSTDIR)/include -I$(OSKIT_INSTDIR)/include/oskit/c \
		-I$(OSKIT_INSTDIR)/include/oskit/threads
endif

ifeq ($(HOST_OS),oskitunix)
ARCH_CFLAGS =	-hosted-oskit $(OSKIT_ARCH_CFLAGS)
ARCH_LIBS =	-loskit_unix $(OSKIT_ARCH_LIBS)
ARCH_PLIBS =	$(addsuffix _p, $(ARCH_LIBS)) -loskit_memfs_p -loskit_gprof
ARCH_DEPINCS =	-I$(OSKIT_INSTDIR)/include -I$(OSKIT_INSTDIR)/include/oskit/c \
		-I$(OSKIT_INSTDIR)/include/oskit/threads
endif

ifeq ($(HOST_OS),linux)
ARCH_CFLAGS += -pthread -D_REENTRANT
endif

ifeq ($(HOST_OS),solaris)
ARCH_CFLAGS +=  -D__solaris__ -D_REENTRANT
ARCH_LIBS +=	-lposix4 -lpthread -lresolv -lsocket
ARCH_PLIBS +=	-lERROR-DontKnowProfilingLibNames-arch.mk # XXX
endif

ifeq ($(HOST_OS),freebsd) 
ARCH_CFLAGS +=	-pthread -D_REENTRANT -DHAVE_SOCKADDR_SA_LEN
endif

#
# Hack to avoid conflicts between -lgcc and -lfreebsd_c(divsi3.o)
# on the arm
#
ifeq ($(HOST_ARCH),arm32)
ifeq ($(HOST_IMPL),shark)
ARCH_CFLAGS +=	-DOSKIT_ARM32_SHARK
endif
ARCH_LIBS +=	-loskit_threads -loskit_dev
ARCH_PLIBS +=	-loskit_threads_p -loskit_dev_p
endif

VPATH=	$(SRCDIR)

###
### Automagic dependency generation stuff
###

# The magic used here for generating the dependency stuff 
# is discussed (and was discovered at):
# http://www.ultranet.com/~pauld/gmake/autodep.html#advanced
#
# The gist of it is: (1) add -MD to the *compile* so that
# the .d file and the .o file are output at the same time.
# (2) Cleanup the .d file to make all the referenced .h's
# into blank rules.  For magical reasons, this makes removing
# a .h file non-fatal.
#

# XXX the gcc test is currently commented out
# ifneq $(filter %gcc, $(CC))
CC_GENDEPEND=-MD
# fixupDependFile: take a .d file and append all of the dependencies
#   as targets w/o rules to the end of the .d
# The following sed script's expressions are (in order):
#   remove the target (all words before a ":" and the ":")
#   remove all leading spaces
#   remove all end-of-line "\"s (and any whitespace around them)
#   add a ":" to the end of every line
define fixupDependFile
echo "#fixuped stuff below" >> "$*.d"
sed	< "$*.d"			\
	-e 's/^[^:]*://'		\
	-e 's/^[ ]*//'			\
	-e 's/[ ]*\\[ ]*$$//'		\
	-e 's/\(.*\)/\1:/'		\
	>> "$*.d"
endef
# else # CC is not GCC
# CC_GENDEPEND=
# define fixupDependFile
# endef
# endif # if CC is not GCC


###
### Generic rules for building .o's and .po's
###

.SUFFIXES:
.SUFFIXES: .o .c .po .i

# How to compile regular object files.
%.o: %.c
	$(SILENT_MAKE_INFORM) "compiling $<"
	rm -f $*.d
	$(CC) $(CC_GENDEPEND) $(CFLAGS) -c  $< -o $@
	$(fixupDependFile)

# How to compile profiled object files.
%.po: %.c
	$(SILENT_MAKE_INFORM) "compiling (with profiling) $<"
	rm -f $*.d
	$(CC) $(CC_GENDEPEND) $(CFLAGS) -pg -DGPROF -c $< -o $@
	$(fixupDependFile)

# For debugging, .i files are cpp output only.
%.i: %.c FORCE
	rm -f $@
	$(CC) $(CFLAGS) -E -c $< > $@

.PHONY: FORCE

#eof
