#!/usr/bin/env python
#
# Python script that acts as a simple way to start up many adoom's and fdoom's
# with arbitrarily complex network topology
#
# NOTE: The rsh remote start support has not been tested.  It may blow up and
# take the entire world with it (though most likely not).
#
# Requires Python 2.0+

# Global options (similar to what's in start.sh)
# This is instantiated at program init, but getGeom is called from startNetwork
# Various commands can also change these options
class Globals:
    def __init__(self):
        self.fdoom="../fdoom/fdoom"
        self.adoom="./adoom"
        self.timeout=20
        self.millispertic=1000/35
        self.xterm="xterm"
        #self.xterm="/home/aclement/misc/Eterm/bin/Eterm"  # Don't do this, utterly beautiful, but things get ugly fast
        if "-g" in sys.argv:
            self.gdb="/home/aclement/janos/src/moab/envs/adoom/igdb -run"
        elif "-g2" in sys.argv:
            self.gdb="/home/aclement/janos/src/moab/envs/adoom/igdb"
        else:
            self.gdb=""
        self.adoomargs=""
        self.fdoomargs=""
        self.log=""
        self.host=""
        # Used for nicely positioning xterms (this could be gotten from
        # xdpyinfo, but, eh)
        self.screenw=1600
        self.screenh=1200

    def getGeom(self,nth):
        """This is called from startNetwork for every xterm that is
        instantiated and should return the -geometry for that xterm"""
        # Get size first
        if len(adooms)+len(fdooms)>12:
            #size=(80,20)
            #font=(5,7)
            size=(80,15)
            font=(6,9)
        elif len(adooms)+len(fdooms)>6:
            size=(80,30)
            font=(6,9)
        else:
            size=(80,40)
            font=(6,12)
        if nth==0:
            font=(6,12)
        # Get offset
        offset=[0,0]
        border=(25,25)
        for x in range(nth):
            offset[0]+=size[0]*font[0]+border[0]
            if offset[0]+size[0]*font[0]+border[0] > self.screenw:
                offset[0]=0
                offset[1]+=size[1]*font[1]+border[1]
                if offset[1]+border[1]>self.screenh:
                    offset=[0,0]
        # -F --geometry  for Eterm
        return "-fn %dx%d -geometry %dx%d+%d+%d" % (font[0],font[1],size[0],size[1],offset[0],offset[1])
        #return "-F %dx%d --geometry %dx%d+%d+%d" % (font[0],font[1],size[0],size[1],offset[0],offset[1])


import cmd,sys,os,types

# List of commands to be executed
cmds=[]

adooms=[]  # Addresses of adooms (after startNetwork call)
fdooms=[]  # Addresses of fdooms
adoompipes=[]  # Pipes to xterms running adooms
fdoompipes=[]  # Pipes to xterms running fdooms

xtermcount=0
def getGeom():
    """Wrapper for globals.getGeom that takes care of the nth argument"""
    global xtermcount
    ret=globals.getGeom(xtermcount)
    xtermcount+=1
    return ret

class adoomCommand:
    def __init__(self, addr):
        self.addr=addr
    def go(self):
        # Get other peers
        others=adooms[:]
        others.remove(self.addr)

        if len(globals.log):
            lname="%s%s" % (globals.log,self.addr)
            try:
                os.unlink(lname)
            except OSError: pass

        bcmd=('%s %s -myaddr %s -timeout %d %s %s -- %s' %
              (globals.gdb,globals.adoom,self.addr,globals.timeout+10,
               globals.adoomargs," ".join(others)," ".join(fdooms)))

        prefix=postfix=""
        if len(globals.host):
            prefix='rsh %s ' % globals.host
            if len(globals.log):
                postfix=' > %s 2>&1 &' % lname
        elif len(globals.log):
            postfix=' > %s 2>&1 &' % lname
        else:
            prefix=('%s %s -title "adoom: %s" -e ' %
                    (globals.xterm,getGeom(),self.addr))

        cmd=prefix+bcmd+postfix

        print cmd
        if not len(globals.log):
            adoompipes.append(os.popen(cmd))
        else:
            os.system(cmd)

class fdoomCommand:
    def __init__(self, faddr, aaddr):
        self.faddr=faddr
        self.aaddr=aaddr
    def go(self):
        # Get other fdoom's
        others=fdooms[:]
        others.remove(self.faddr)

        if len(globals.log):
            lname="%s%s" % (globals.log,self.faddr)
            try:
               os.unlink(lname)
            except OSError: pass

        bcmd=('%s %s -myaddr %s -millispertic %d -timeout %d -adoomserver %s %s %s' %
              (globals.gdb,globals.fdoom,
               self.faddr,globals.millispertic,globals.timeout,self.aaddr,
               globals.fdoomargs," ".join(others)))

        prefix=postfix=""
        if len(globals.host):
            prefix='rsh %s' % globals.host
            if len(globals.log):
                postfix=' > %s 2>&1 &' % lname
        elif len(globals.log):
            postfix=' > %s 2>&1 &' % lname
        else:
            prefix=('%s %s -title "fdoom: %s" -e ' %
                    (globals.xterm,getGeom(),self.faddr))

        cmd=prefix+bcmd+postfix

        print cmd
        if not len(globals.log):
            fdoompipes.append(os.popen(cmd))
        else:
            os.system(cmd)

class waitCommand:
    def __init__(self, time):
        self.time=time
    def go(self):
        cmd=('sleep %d' % self.time)
        print cmd
        os.system(cmd)

class setCommand:
    def __init__(self, var, val):
        self.var=var
        self.val=val
    def go(self):
        print "# Setting "+`self.var`+" to "+`self.val`
        exec ("globals.%s = %s" % (self.var,`self.val`))
            
    
def startNetwork():
    # Get all adoom addresses
    for x in cmds:
        if isinstance(x,adoomCommand):
            adooms.append(x.addr)
            
    # Get all fdoom addresses (and check for collisions)
    for x in cmds:
        if isinstance(x,fdoomCommand):
            if not x.aaddr in adooms:
                print "*** Error: fdoom server address %s is not an adoom" % x.aaddr
                return
            elif x.faddr in fdooms:
                print "*** Error: fdoom address collision %s" % x.faddr
                return
            else:
                fdooms.append(x.faddr)

    print "Adooms: %s" % " ".join(adooms)
    print "Fdooms: %s" % " ".join(fdooms)

    # Get global setup
    global globals
    
    # Execute commands
    for x in cmds:
        if x.go():
            return

    waitOnChildren()

def waitOnChildren():
    """Waits on all child processes, or kills them if a KeyboardInterrupt is
    thrown"""
    allpipes=adoompipes+fdoompipes
    while len(allpipes):
        try:
            print "Waiting on child"
            os.wait()
            # Process must have finished, go to next
            allpipes=allpipes[1:]
        except KeyboardInterrupt:
            print "KeyboardInterrupt caught; all children will be nuked"
            sys.exit(1)


class MyCmd(cmd.Cmd):
    """Command interpreter"""
    def __init__(self):
        if sys.stdin.isatty():
            self.prompt="> "
        else:
            self.prompt=""
        self.ruler=""
    def emptyline(self):
        # Ignore empty lines, but don't do default cmd behavior
        return
    def do_EOF(self,args):
        print
        sys.exit(0)
    
    def help_adoom(self):
        print """Creates an adoom node in the network.
Syntax: adoom <addr>
Where addr is the full address of the adoom server (ie 127.0.0.1:28000)"""
    def do_adoom(self,args):
        args=args.split()
        if len(args)!=1:
            print "*** Invalid syntax: adoom (One arg expected)"
        elif args[0] in adooms:
            print "*** Error: Repeated adoom server address "+addr
        else:
            cmds.append(adoomCommand(args[0]))
            
    def help_fdoom(self):
        print """Creates an fdoom node in the network.
Syntax: fdoom <addr> <adoomaddr>
Where addr is the full address of the fdoom client and adoomaddr is the
address of the adoom server to connect to"""
    def do_fdoom(self,args):
        args=args.split()
        if len(args)!=2:
            print "*** Invalid syntax: fdoom (Two args expected)"
        else:
            cmds.append(fdoomCommand(args[0],args[1]))

    def help_wait(self):
        print """Pauses the network creation for n seconds.  Useful in scripts
to delay the creation of nodes.
Syntax: wait <n>
Where n is the number of seconds to wait or a Python expression that evaluates
to a number"""
    def do_wait(self,args):
        if len(args)==0:
            print "*** Invalid syntax: wait (Args expected)"
        else:
            wait=eval(args)
            if isinstance(wait,types.IntType):
                cmds.append(waitCommand(wait))
            else:
                print "*** Invalid syntax: wait (Number expected)"

    def help_timeout(self):
        print """Sets the timeout to be passed to the nodes on start
Syntax: timeout <secs>
Where secs is the number of seconds to be passed as -timeout or a Python
expression that evaluates to a number
Default: """+`globals.timeout`
    def do_timeout(self,args):
        if len(args)==0:
            timeout=20
        else:
            timeout=eval(args)
            if not isinstance(timeout,types.IntType):
                print "*** Invalid syntax: timeout (Number expected)"
                return
        cmds.append(setCommand("timeout",timeout))

    def help_millispertic(self):
        print """Sets the millispertic to be passed to the nodes on start
Syntax: millispertic <mps>
Where mps is the value to be passed as -millispertic or a Python expression
that evaluates to a number
Default: """+`globals.millispertic`
    def do_millispertic(self,args):
        if len(args)==0:
            millispertic=1000/25
        else:
            millispertic=eval(args)
            if not isinstance(millispertic,types.IntType):
                print "*** Invalid syntax: millispertic (Number expected)"
                return
        cmds.append(setCommand("millispertic",millispertic))

    def help_gdb(self):
        print """Sets the command to be used for gdb
Syntax: gdb <gdb>
Where gdb is the command to be used
Default: """+globals.gdb
    def do_gdb(self,args):
        cmds.append(setCommand("gdb",args))

    def help_adoomargs(self):
        print """Sets the arguments to be passed to adoom servers on start
Syntax: adoomargs <args>
Where args are the arguments to pass
Default: """+globals.adoomargs
    def do_adoomargs(self,args):
        cmds.append(setCommand("adoomargs",args))

    def help_fdoomargs(self):
        print """Sets the arguments to be passed to fdoom clients on start
Syntax: fdoomargs <args>
Where args are the arguments to pass
Default: """+globals.fdoomargs
    def do_fdoomargs(self,args):
        cmds.append(setCommand("fdoomargs",args))

    def help_adoombin(self):
        print """Sets the name and path of the adoom binary to use
Syntax: adoombin <file>
Where file is the filename and path of the binary
Default: """+globals.adoombin
    def do_adoombin(self,args):
        cmds.append(setCommand("adoom",args))

    def help_fdoombin(self):
        print """Sets the name and path of the fdoom binary to use
Syntax: fdoombin <file>
Where file is the filename and path of the binary
Default: """+globals.fdoombin
    def do_fdoombin(self,args):
        cmds.append(setCommand("fdoom",args))

    def help_log(self):
        print """Sets the logfile prefix.
Syntax: log <prefix>
Where prefix is the prefix for the log file names.  It will be followed by the
address of the node.  If prefix is not given, logging is disabled."""
    def do_log(self,args):
        cmds.append(setCommand("log",args))

    def help_host(self):
        print """Sets the host on which to run nodes.  This attempts to use
rsh to access the given host
Syntax: host <host>
Where host is the host to run on.  If host is not given, the nodes will be run
on the local system"""
    def do_host(self,args):
        cmds.append(setCommand("host",args))
            
    def help_go(self):
        print """Runs the network as the topology stands.  Note that the
topology cannot be changed once the network has started.  Also, *don't* forget
to put a "go" at the end of a piped script.
Syntax: go"""
    def do_go(self,args):
        if len(args):
            print "*** Invalid syntax: go"
        else:
            startNetwork()
            sys.exit(0)

globals=Globals()
MyCmd().cmdloop()
