/*
 * stats.c
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <stdio.h> // printf

#include <nodeos/an_event.h>

#include "stats.h"
#include "main.h"
#include "peer.h"

AdoomStats adoomStats;


static an_event_t statsEvent;
static void printStats(an_event_t *even, void *arg);

void
statsInit(void)
{
	memset(&adoomStats, 0, sizeof adoomStats);
}

void
registerStatsTrigger(void)
{
	an_error_t err;
	an_timespec_t gap;
	
	an_event_create(&statsEvent);
	
	gap.tv_sec = options.statsprintgap;
	gap.tv_nsec = 0;
	
	err = an_event_schedule(&statsEvent,
				&gap, AN_EVTYPE_PERIODIC,
				printStats, NULL);
	assert(err == 0);
}

void
shutdownStats(void)
{
	an_error_t err;
	
	err = an_event_deschedule(&statsEvent, NULL, NULL);
	assert(err == 0);
	err = an_event_destroy(&statsEvent, NULL, NULL);
	assert(err == 0);
}

static void
doPrintStats(void)
{
	printf("clientsent=%ld; clientrecv=%ld; peersent=%ld; peerrecv=%ld; #clients=%d; #peers=%d;\n",
	       adoomStats.clientPacketSentCount,
	       adoomStats.clientPacketRecvCount,
	       adoomStats.peerPacketSentCount,
	       adoomStats.peerPacketRecvCount,
	       clientCount(),
	       peerCount());
}

static void
printStats(an_event_t *event, void *arg)
{
	doPrintStats();
}

void
printFinalStats(void)
{
	// Option info
#define POPT_D(VAR) printf("%32s: %d\n", #VAR, options.VAR)
	POPT_D(timeout);
	POPT_D(statsprintgap);
	POPT_D(setuppoll);
	printf("%32s: %d\n", "ticfillsize", ticfillsize);

#define PSTAT_U32(VAR) printf("%32s: %lu\n", #VAR, adoomStats.VAR)
	PSTAT_U32(clientPacketRecvCount);
	PSTAT_U32(clientPacketSentCount);
	PSTAT_U32(peerPacketRecvCount);
	PSTAT_U32(peerPacketSentCount);
	PSTAT_U32(connectPacketFromActive);
	PSTAT_U32(outOfOrderClientPacket);
	PSTAT_U32(peerLockAcquire);
	PSTAT_U32(peerLockRelease);
	PSTAT_U32(clientLockAcquire);
	PSTAT_U32(clientLockRelease);
	PSTAT_U32(blockOnPoolMutex);
	PSTAT_U32(poolWaitOnPbuf);
	PSTAT_U32(poolLockAcquire);
	PSTAT_U32(poolLockRelease);
}
