/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_event.h>
#include <nodeos/an_info.h>
#include <nodeos/lib/utils.h>

#include <nodeos/impl/an_gprof.h>	/* XXX */

#include <nodeos/lib/anlib_chan.h>

#define RUNFOR		1*60*60*5	/* run for this many seconds */
#define INIF		0	/* interface to use for input; i.e. "if<INIF>" */
#define OUTIF		0	/* ditto for output */
#define MAXBUFS		64
#define MAX_ARPBUFS 2
#define MAX_ARPQBUFS 2

static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };


static void ee_die(an_event_t *event, void *arg);
static void setup_channels(void);

/* source enet addr on INIF (filled in by nodeos) */
an_ethaddrspec_t myaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};

static an_pbuf_t *bufs[2 * MAXBUFS], bufs_data[2 * MAXBUFS];
static an_pbuf_t *arp_bufs[MAX_ARPBUFS], arp_bufs_data[MAX_ARPBUFS];
static an_pbuf_t *arpq_bufs[MAX_ARPQBUFS], arpq_bufs_data[MAX_ARPQBUFS];

static void ee_udprecv(an_pbuf_t *packet, void *arg, int msize);
static void ee_udprecv2(an_pbuf_t *packet, void *arg, int msize);

static an_chan_t inchan_data, inchan2_data; 
static anlib_chan_t *inchan, *inchan2;
static anlib_chan_t *outchan; 
static an_chan_t outchan_data;

static int npackets, npackets2, chandead;
static unsigned long nbytes;
static unsigned long nbytes2;
static an_event_t eventmem;

static void
ee_init(void *arg)
{
	an_initargs_t *initargs;
	
	/*
	 * Parse command line arguments.
	 *    socktest [blow [blowhostip]]
	 */
	initargs = (an_initargs_t*)arg;
	assert(initargs);

	/*
	 * Setup channels
	 */
	setup_channels();

	printf("waiting to die...\n");

	an_gprof_start(initargs->argv[0]);
	return;
}

static void
setup_channels()
{
	int	i;
	an_error_t err;
	an_infoif_t info[8];
	int nif = 8;
	char inifname[8], outifname[8];
	an_chanproto_t proto;
	an_netspec_t net;
	anlib_chanaddr_t addr;
	an_event_t *event_end;
	an_timespec_t etime;
	int ret;
	
	err = an_info_getifs(info, &nif);
	assert(err == 0);

	/*
	 * Find my ETH interface address
	 */
	assert(nif >= INIF);
	assert(info[INIF].type == ANIF_ETH);
	memcpy(myaddr.daddr, info[INIF].addr.eth.daddr, sizeof myaddr.daddr);
	myaddr.type = htons(myaddr.type);
	printf("opened if %02x:%02x:%02x:%02x:%02x:%02x\n",	myaddr.daddr[0], 
			myaddr.daddr[1], myaddr.daddr[2], myaddr.daddr[3], 
			myaddr.daddr[4], myaddr.daddr[5]);

	sprintf(inifname, "eth%d", INIF);
	assert(nif >= OUTIF);
	assert(info[OUTIF].type == ANIF_ETH);
	sprintf(outifname, "eth%d", OUTIF);

	/*
	 * Setup an event to time us out in RUNFOR seconds
	 */
	event_end = an_event_create (&eventmem);
	assert (event_end);
	
	etime.tv_sec = RUNFOR;
	etime.tv_nsec = 0;
	err = an_event_schedule(event_end, &etime, AN_EVTYPE_ONESHOT_RELATIVE,
							ee_die, 0 );
	assert(err == 0);
	printf("Timer started, running for %d seconds\n", RUNFOR);

	/*
	 * Buffers for input side.
	 */
	for (i = 0; i < 2 * MAXBUFS; i++) {
		bufs[i] = an_pbuf_create(&bufs_data[i], AN_PBUF_DEFAULT,
					    0, 0);
		assert(bufs[i] != 0);
	}

	sprintf(proto.data, "%s/ip/udp", inifname);
	proto.len = strlen(proto.data);
	net.maxthreads = 1;
	net.buffers.npbufs = MAXBUFS;
	net.buffers.pbufs = bufs;
	sprintf(addr.raw.data, "::%04x/::%u/5678:", ETHERTYPE_IP, IPPROTO_UDP);
	addr.raw.len = strlen(addr.raw.data);
	
	inchan = anlib_inchan_create(&inchan_data, an_domain_current(), 0, &proto,
									&addr, &net, ee_udprecv, 0);	

    assert(inchan);
	printf("receiving UDP/IP "
	       "on %s to %02x:%02x:%02x:%02x:%02x:%02x\n",
	       inifname,
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5]);

	net.maxthreads = 1;
	net.buffers.npbufs = MAXBUFS;
	net.buffers.pbufs = bufs + MAXBUFS;
	sprintf(addr.raw.data, "::%04x/::%u/", ETHERTYPE_IP, IPPROTO_UDP);
	addr.raw.len = strlen(addr.raw.data);

	inchan2 = anlib_inchan_create(&inchan2_data, an_domain_current(), 0, &proto,
			&addr, &net, ee_udprecv2, 0);
    assert(inchan2);
	sprintf(proto.data, "udpip/arp/%s", outifname);
	proto.len = strlen(proto.data);
	sprintf(addr.raw.data, "192.168.1.2:5678:192.168.1.3:5678/192.168.1.2:"
			"00d0b71007dc/");
	addr.raw.len = strlen(addr.raw.data);
	net.bandwidth = 1;

	/*
	 * Buffers for arp responses 
	 */
	for (i = 0; i < MAX_ARPBUFS; i++) {
		arp_bufs[i] = an_pbuf_create(&arp_bufs_data[i], AN_PBUF_DEFAULT,
					    0, 0);
		assert(arp_bufs[i] != 0);
	}
	net.buffers.npbufs = MAX_ARPBUFS;
	net.buffers.pbufs = arp_bufs;
	net.maxthreads = 1;

	outchan = anlib_outchan_create(&outchan_data, an_domain_current(), &proto,
			&addr, &net);
    assert(outchan);
	sprintf(addr.raw.data, "192.168.1.2:00d0b71007dc");
	addr.raw.len = strlen(addr.raw.data);	
	
	strcpy(proto.data, "eth0");
	proto.len = strlen(proto.data);

	for (i = 0; i < MAX_ARPQBUFS; i++) {
		arpq_bufs[i] = an_pbuf_create(&arpq_bufs_data[i], AN_PBUF_DEFAULT,
						0, 0);
		assert(arpq_bufs[i] != 0);
	}
	net.buffers.npbufs = MAX_ARPQBUFS;
	net.buffers.pbufs = arpq_bufs;
	net.maxthreads = 1;
	ret = anlib_arpResponder_create(an_domain_current(), &addr, &proto, &net);
	if (!ret)
		printf("Unable to create arp responder\n");
    assert(ret);
}

static void
ee_udprecv(an_pbuf_t *pbuf, void *arg, int msize)
{
	int psize = an_pbuf_size(pbuf);
	if (chandead)
		return;
	npackets++;
	nbytes += psize;
	anlib_outchan_send(outchan, pbuf);
}

static void
ee_udprecv2(an_pbuf_t *pbuf, void *arg, int msize)
{
	int psize = an_pbuf_size(pbuf);

	if (chandead)
		return;

	npackets2++;
	nbytes2 += psize;
	anlib_outchan_send(outchan, pbuf);
}


extern void print_arp_stats(void);
static void
ee_die(an_event_t *event, void *arg)
{
	an_chanproto_t proto;
	chandead = 1;

	printf("Done: %lu bytes, %d packets\n", nbytes, npackets);
	printf("Done2: %lu bytes, %d packets\n", nbytes2, npackets2);

	moab_chan_printstats(inchan->raw_chan, 1, "UDP in", RUNFOR, 0);
	moab_chan_printstats(outchan->raw_chan, 1, "UDP out", RUNFOR, 0);
	print_arp_stats();
	strcpy(proto.data, "eth0");
	proto.len = strlen(proto.data);
	anlib_arpResponder_destroy(&proto);
	anlib_chan_destroy(inchan);
	anlib_chan_destroy(inchan2);
	anlib_chan_destroy(outchan);
	an_domain_destroy(an_domain_current());
}
