/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to test channels
 */

#undef CHAN_ETH
#define CHAN_ETH_DEV 0

#define VERBOSE
#undef TESTHANDOFF

#define RUNFOR	20

#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mempool.h>
#include <nodeos/an_pbuf.h>

static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };

static void setup_channels(void);
static void ee_packet1(an_pbuf_t *pbuf, void *arg, int msize);
static void ee_packet100(an_pbuf_t *pbuf, void *arg, int msize);
static an_chan_t *chan1, chan1_data, *chan100, chan100_data;
static an_pbuf_t *buf1, buf1_data, *buf100, buf100_data;
#ifdef TESTHANDOFF
static void ee_packet1000(an_pbuf_t *pbuf, void *arg, int msize);
static an_chan_t *chan1000, chan1000_data;
static an_pbuf_t *buf1000, buf1000_data;
#endif

static struct chanattr {
	an_chanaddr_t addr;
	an_chanproto_t proto;
	an_netspec_t net;
	an_pbuf_t *bufs[1];
} attr;
static an_domain_t *domain;

#ifdef CHAN_ETH
#include <nodeos/an_info.h>

static void ee_eth(void);
static void ee_unipacket(an_pbuf_t *pbuf, void *arg, int msize);
static void ee_otherpacket(an_pbuf_t *pbuf, void *arg, int msize);
static an_infoif_t info[8];
struct ethstuff {
	an_chan_t *eth;
	an_chan_t eth_data;
	an_demuxkey_t *key;
	an_demuxkey_t key_data;
	an_pbuf_t *pbuf;
	an_pbuf_t pbuf_data;
	an_infoif_t *info;
};
#endif

int
ee_main(int argc, const char **argv)
{
	domain = an_domain_current();
	printf("%p/%p: Channel test EE started\n",
	       domain, an_thread_current());

	/*
	 * Setup channels
	 */
	setup_channels();

	/*
	 * No longer need this thread
	 */
	return 0;
}

static void
setup_channels(void)
{
	memset(&attr, 0, sizeof attr);

#ifdef CHAN_ETH
	ee_eth();
#endif

	/*
	 * Create packet buffers
	 */
	buf1 = an_pbuf_create(&buf1_data, AN_PBUF_CLOCK, 0, 0);
	assert(buf1 != 0);
	buf100 = an_pbuf_create(&buf100_data, AN_PBUF_CLOCK, 0, 0);
	assert(buf100 != 0);

	strcpy(attr.proto.data, "clock");
	attr.proto.len = strlen(attr.proto.data);
	attr.addr.clock.freq = 100;
	attr.net.maxthreads = 1;
	attr.bufs[0] = buf100;
	attr.net.buffers.npbufs = 1;
	attr.net.buffers.pbufs = attr.bufs;
	chan100 = an_inchan_create(&chan100_data, domain, 0,
				   &attr.proto, &attr.addr, &attr.net,
				   ee_packet100, &chan100_data);
	assert(chan100 != 0);

	attr.addr.clock.freq = 1;
	attr.net.maxthreads = 1;
	attr.bufs[0] = buf1;
	attr.net.buffers.npbufs = 1;
	attr.net.buffers.pbufs = attr.bufs;
	chan1 = an_inchan_create(&chan1_data, domain, 0,
				 &attr.proto, &attr.addr, &attr.net,
				 ee_packet1, &chan1_data);
	assert(chan1 != 0);

#ifdef TESTHANDOFF
	buf1000 = an_pbuf_create(&buf1000_data, AN_PBUF_CLOCK, 0, 0);
	assert(buf1000 != 0);
	attr.addr.clock.freq = 1000;
	attr.net.maxthreads = 1;
	attr.bufs[0] = buf1000;
	attr.net.buffers.npbufs = 1;
	attr.net.buffers.pbufs = attr.bufs;
	chan1000 = an_inchan_create(&chan1000_data, domain, 0,
				    &attr.proto, &attr.addr, &attr.net,
				    ee_packet1000, &chan1000_data);
	assert(chan1000 != 0);
#endif
}

static int inpackets;

static void
ee_packet1(an_pbuf_t *pbuf, void *arg, int msize)
{
	assert(arg == chan1);

	inpackets++;
	an_inchan_addpbuf(chan1, pbuf);
}

static void
ee_packet100(an_pbuf_t *pbuf, void *arg, int msize)
{
	static int count;
	unsigned int *ts = an_pbuf_data(pbuf);
	an_timespec_t tspec;
	
	assert(arg == chan100);
	assert(an_pbuf_size(pbuf) == sizeof *ts);

	an_gettime(&tspec);
	printf("%p/%p: packet %p@%u: got %d packets at %u seconds\n",
	       an_domain_current(), an_thread_current(), pbuf, *ts, inpackets,
	       (unsigned int)tspec.tv_sec);
#ifdef TESTHANDOFF
	printf("  Handing off...\n");
	an_inchan_handoff(chan1000, pbuf);
#endif
	an_inchan_addpbuf(chan100, pbuf);
	if (++count == RUNFOR) {
		printf("killing 1-tick timer channel %p\n", chan1);
		if (chan1)
			an_chan_destroy(chan1);
		printf("killing 100-tick timer channel %p\n", chan100);
		if (chan100)
			an_chan_destroy(chan100);
		printf("killing our domain %p\n", an_domain_current());
		an_domain_destroy(an_domain_current());
	}
}

#ifdef TESTHANDOFF
static void
ee_packet1000(an_pbuf_t *pbuf, void *arg, int msize)
{
	unsigned int *ts = an_pbuf_data(pbuf);
	
	assert(arg == chan1000);
	assert(an_pbuf_size(pbuf) == sizeof *ts);

	printf("  got handoff packet %p@%u\n", pbuf, *ts);
	an_inchan_addpbuf(chan1000, pbuf);
}
#endif

#ifdef CHAN_ETH
static void
ee_eth(void)
{
	int nif = 8;
	int i;
	an_error_t err;
	struct ethstuff *eth;

	/*
	 * Find all interfaces.
	 */
	err = an_info_getifs(info, &nif);
	assert(err == 0);

	/*
	 * For each interface we setup two channels:
	 *	- one with dest specified to match unicast packets for the IF
	 *	- one with no addr spec to match everything
	 * The latter channel should be lower priority (based on length of
	 * match criteria) and should only catch those packets not caught by
	 * the former (i.e., broadcast packets).
	 */
	for (i = 0; i < nif; i++) {
		assert(info[i].type == ANIF_ETH);
#ifdef CHAN_ETH_DEV
		if (info[i].id != CHAN_ETH_DEV)
			continue;
#endif

#ifdef VERBOSE
		printf("if%d: %s: %02x:%02x:%02x:%02x:%02x:%02x\n",
		       info[i].id, info[i].pname,
		       info[i].addr.eth.daddr[0], info[i].addr.eth.daddr[1],
		       info[i].addr.eth.daddr[2], info[i].addr.eth.daddr[3],
		       info[i].addr.eth.daddr[4], info[i].addr.eth.daddr[5]);
#endif

		/*
		 * Unicast packets
		 */
		eth = an_mempool_alloc(an_mempool_current(), sizeof *eth);
		assert(eth != 0);
		eth->info = &info[i];

		eth->key = an_demuxkey_create(&eth->key_data, domain);
		assert(eth->key != 0);
		err = an_demuxkey_add(eth->key, 6, info[i].addr.eth.daddr, 0);
		assert(err == 0);

		memset(&attr, 0, sizeof attr);
		strncpy(attr.proto.data, info[i].pname, sizeof info[i].pname);
		attr.proto.len = strlen(attr.proto.data);
		attr.net.maxthreads = 1;
		attr.bufs[0] = eth->pbuf = an_pbuf_create(&eth->pbuf_data,
							  AN_PBUF_ETH, 0, 0);
		attr.net.buffers.npbufs = 1;
		attr.net.buffers.pbufs = attr.bufs;
		eth->eth = an_inchan_create(&eth->eth_data, domain, eth->key,
					    &attr.proto, &attr.addr, &attr.net,
					    ee_unipacket, eth);
		assert(eth->eth != 0);

		/*
		 * Other packets
		 */
		eth = an_mempool_alloc(an_mempool_current(), sizeof *eth);
		assert(eth != 0);
		eth->info = &info[i];

		eth->key = an_demuxkey_create(&eth->key_data, domain);
		assert(eth->key != 0);

		memset(&attr, 0, sizeof attr);
		strncpy(attr.proto.data, info[i].pname, sizeof info[i].pname);
		attr.proto.len = strlen(attr.proto.data);
		attr.net.maxthreads = 1;
		attr.bufs[0] = eth->pbuf = an_pbuf_create(&eth->pbuf_data,
							  AN_PBUF_ETH, 0, 0);
		attr.net.buffers.npbufs = 1;
		attr.net.buffers.pbufs = attr.bufs;
		eth->eth = an_inchan_create(&eth->eth_data, domain, eth->key,
					    &attr.proto, &attr.addr, &attr.net,
					    ee_otherpacket, eth);
		assert(eth->eth != 0);
	}
}

static void
ee_unipacket(an_pbuf_t *pbuf, void *arg, int msize)
{
	struct ethstuff *eth = arg;
	int size = an_pbuf_size(pbuf);
	struct {
		unsigned char daddr[6];
		unsigned char saddr[6];
		short type;
	} *hdr = an_pbuf_data(pbuf);

	assert(eth->info != 0);
	assert(size >= sizeof *hdr);
	assert(msize == 6);

	printf("%s(1): "
	       "%02x:%02x:%02x:%02x:%02x:%02x -> %02x:%02x:%02x:%02x:%02x:%02x"
	       ": %x\n",
	       eth->info->pname,
	       hdr->saddr[0], hdr->saddr[1], hdr->saddr[2],
	       hdr->saddr[3], hdr->saddr[4], hdr->saddr[5],
	       hdr->daddr[0], hdr->daddr[1], hdr->daddr[2],
	       hdr->daddr[3], hdr->daddr[4], hdr->daddr[5], ntohs(hdr->type));
	an_inchan_addpbuf(eth->eth, pbuf);
}

static void
ee_otherpacket(an_pbuf_t *pbuf, void *arg, int msize)
{
	struct ethstuff *eth = arg;
	int size = an_pbuf_size(pbuf);
	struct {
		unsigned char daddr[6];
		unsigned char saddr[6];
		short type;
	} *hdr = an_pbuf_data(pbuf);

	assert(eth->info != 0);
	assert(size >= sizeof *hdr);
	assert(msize == size);

	printf("%s(2): "
	       "%02x:%02x:%02x:%02x:%02x:%02x -> %02x:%02x:%02x:%02x:%02x:%02x"
	       ": %x\n",
	       eth->info->pname,
	       hdr->saddr[0], hdr->saddr[1], hdr->saddr[2],
	       hdr->saddr[3], hdr->saddr[4], hdr->saddr[5],
	       hdr->daddr[0], hdr->daddr[1], hdr->daddr[2],
	       hdr->daddr[3], hdr->daddr[4], hdr->daddr[5], ntohs(hdr->type));
	an_inchan_addpbuf(eth->eth, pbuf);
}
#endif

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
}

