/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
 
/*
 * EE tests click library with a NAT config. 
 */

/*
 * Variables ptentially need to be changed: INTERN, EXTERN(#define'd below)
 * They represent the interfaces of the router connected to the 
 * internal and external network respectively.
 * Nothing else needs to be changed/edited, it should just work.
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_event.h>
#include <nodeos/an_time.h>
#include <nodeos/an_info.h>
#include <nodeos/impl/an_gprof.h>
#include <nodeos/lib/utils.h>

#include <nodeos/lib/click.hh>

#define MAXBUFS 8

#define INTERN 1 	// Interface connected to the internal network
#define EXTERN 2	// interface connected to the outside world

static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };

static void setup_channels(void);

static an_chan_t *intern_ethinchan, intern_ethinchan_data, 
				 *intern_ethoutchan, intern_ethoutchan_data,
				 *extern_ethinchan, extern_ethinchan_data,
				 *extern_ethoutchan, extern_ethoutchan_data;

static an_pbuf_t internbuf_data[MAXBUFS], externbuf_data[MAXBUFS];

anlib_clickgraphdummy_t graph_data;
anlib_clickgraph_t *graph;
static char click_config[];
static char config[2048];

int
ee_main(int argc, const char ** argv)
{
	assert(argc >= 1);
	printf("%p/%p: %s started\n",
		       an_domain_current(), an_thread_current(), argv[0]);
	
	setup_channels();
	printf("NAT ready...\n");
	
	an_gprof_start(argv[0]);
	return 0;
}

static void
setup_channels()
{
	an_error_t err;
	an_netspec_t net;
	an_infoif_t info[8];
	int i, nif = 8;
	an_domain_t *domain;
	
	struct {
		an_chanaddr_t addr;
		an_chanproto_t proto;
		an_pbuf_t *bufs[MAXBUFS];
		an_chanrecvfunc_t func;
		void *funcarg;
	} intern_attr, extern_attr;

	
	memset(&intern_attr, 0, sizeof intern_attr);
	memset(&extern_attr, 0, sizeof extern_attr);

	err = an_info_getifs(info, &nif);
	assert(err == 0);
	assert(info[INTERN].type == ANIF_ETH);
	assert(info[EXTERN].type == ANIF_ETH);

	sprintf(config, click_config, 
	       info[INTERN].addr.eth.daddr[0], info[INTERN].addr.eth.daddr[1], info[INTERN].addr.eth.daddr[2],
	       info[INTERN].addr.eth.daddr[3], info[INTERN].addr.eth.daddr[4], info[INTERN].addr.eth.daddr[5],
	       info[EXTERN].addr.eth.daddr[0], info[EXTERN].addr.eth.daddr[1], info[EXTERN].addr.eth.daddr[2],
	       info[EXTERN].addr.eth.daddr[3], info[EXTERN].addr.eth.daddr[4], info[EXTERN].addr.eth.daddr[5]);
	    	
	graph = anlib_clickgraph_create(&graph_data, config);
	assert(graph != 0);
	
	domain = an_domain_current();
	/* Create raw eth channel for the interface to internal network */
	for (i = 0; i < MAXBUFS; i++) {
		intern_attr.bufs[i] = an_pbuf_create(&internbuf_data[i], AN_PBUF_ETH, 
									0, 0);
		assert (intern_attr.bufs[i] != 0);
	}

	err = anlib_clickgraph_lookupInput(graph, "intin", &intern_attr.func,
			&intern_attr.funcarg);
	assert(err ==0);

	strncpy(intern_attr.proto.data, info[INTERN].pname, sizeof
			info[INTERN].pname);
	intern_attr.proto.len = strlen(intern_attr.proto.data);

	net.buffers.pbufs = intern_attr.bufs;
	net.buffers.npbufs = MAXBUFS;
	net.maxthreads = 1;

	intern_ethinchan = an_inchan_create(&intern_ethinchan_data, domain, 0, 
						&intern_attr.proto, &intern_attr.addr, &net, intern_attr.func, intern_attr.funcarg);
	
	/* Create raw ethernet channel for the interface to outside world */
	for (i = 0; i < MAXBUFS; i++) {
		extern_attr.bufs[i] = an_pbuf_create(&externbuf_data[i], AN_PBUF_ETH, 
									0, 0);
		assert (extern_attr.bufs[i] != 0);
	}

	err = anlib_clickgraph_lookupInput(graph, "extin", &extern_attr.func,
			&extern_attr.funcarg);
	assert(err ==0);

	strncpy(extern_attr.proto.data, info[EXTERN].pname, sizeof
			info[EXTERN].pname);
	extern_attr.proto.len = strlen(extern_attr.proto.data);

	net.buffers.pbufs = extern_attr.bufs;
	net.buffers.npbufs = MAXBUFS;
	net.maxthreads = 1;

	extern_ethinchan = an_inchan_create(&extern_ethinchan_data, domain, 0, 
						&extern_attr.proto, &extern_attr.addr, &net, extern_attr.func, extern_attr.funcarg);
	
	/* Now create raw output channel for internal network */
	memset(&intern_attr, 0, sizeof intern_attr);
	strncpy(intern_attr.proto.data, info[INTERN].pname, 
											sizeof info[INTERN].pname);
	intern_attr.proto.len = strlen(intern_attr.proto.data);
	memset(&intern_attr.addr, 0, sizeof intern_attr.addr);
	net.bandwidth = 1;
	intern_ethoutchan = an_outchan_create(&intern_ethoutchan_data, domain,
				       &intern_attr.proto, &intern_attr.addr, &net);

	/* Create raw output channel for external network */
	memset(&extern_attr, 0, sizeof extern_attr);
	strncpy(extern_attr.proto.data, info[EXTERN].pname, 
											sizeof info[EXTERN].pname);
	extern_attr.proto.len = strlen(extern_attr.proto.data);
	memset(&extern_attr.addr, 0, sizeof extern_attr.addr);
	net.bandwidth = 1;
	extern_ethoutchan = an_outchan_create(&extern_ethoutchan_data, domain,
				       &extern_attr.proto, &extern_attr.addr, &net);


	/* Attach the output from the click graph to output channels */

	err = anlib_clickgraph_attachOutput(graph, "intout", an_outchan_send,
			intern_ethoutchan);

	assert(err == 0);

	err = anlib_clickgraph_attachOutput(graph, "extout", an_outchan_send,
			extern_ethoutchan);
	assert(err == 0);

	/* Guess we are done. It might just work... */
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "clickMazuNATEE" };

	status = ee_main(argc, argv);
}


static char click_config[] = 
	"AddressInfo(
  		intern 	10.0.0.1	10.0.0.0/8	%02x:%02x:%02x:%02x:%02x:%02x,
  		extern	209.6.198.213	209.6.198.0/24	%02x:%02x:%02x:%02x:%02x:%02x
	);"


// DEVICE SETUP

"from_extern :: FromInChan(extin);
to_extern :: ToOutChan(extout);
from_intern :: FromInChan(intin);
to_intern :: ToOutChan(intout);"

// ARP MACHINERY

"extern_arp_class, intern_arp_class
	:: Classifier(12/0806 20/0001, 12/0806 20/0002, 12/0800, -);
intern_arpq :: ARPQuerier(intern);
extern_arpq :: ARPQuerier(extern); 

from_extern -> extern_arp_class;
extern_arp_class[0] -> ARPResponder(extern)	
	-> to_extern;
extern_arp_class[1] -> [1]extern_arpq;	
extern_arp_class[3] -> Discard;

from_intern -> intern_arp_class;
intern_arp_class[0] -> ARPResponder(intern)
	-> to_intern;
intern_arp_class[1] -> [1]intern_arpq; 	
intern_arp_class[3] -> Discard;"


// REWRITERS

// This patten would rewrite the source address and port number.
"IPRewriterPatterns(to_world_pat extern 32768-65535 - -);"

"rw :: IPRewriter(
		 pattern to_world_pat 0 1, 
		 nochange 2
		 );"

// OUTPUT PATH

"ip_to_extern :: GetIPAddress(16) 
      -> CheckIPHeader"
      "-> [0]extern_arpq
	  -> to_extern;"

"ip_to_intern :: GetIPAddress(16) 
      -> CheckIPHeader
      -> [0]intern_arpq
      -> to_intern;"


// to outside world or gateway from inside network
"rw[0] ->  
  ip_to_extern_class :: IPClassifier(dst host intern, -);
  ip_to_extern_class[0] -> Discard ;
  ip_to_extern_class[1] -> ip_to_extern;"

"rw[1] -> ip_to_intern;" // Reply domains to internal network
"rw[2] -> Discard;" // Packets from outside that do not have an established
				  // mapping are discarded.

// HANDLE IP PACKETS FROM OUTSIDE

"ip_from_extern :: IPClassifier(dst host extern,
			-);"

"my_ip_from_extern :: IPClassifier(dst tcp port < 32768 or dst udp port < 32768,
			(dst tcp port >= 32768 and dst tcp port <= 65535) or 
			(dst udp port >= 32768 and dst udp port <= 65535),  
			tcp or udp,
			-);"

"extern_arp_class[2] -> Strip(14)
  	-> CheckIPHeader()
	-> ip_from_extern;"

"ip_from_extern[0] -> my_ip_from_extern; 	
	my_ip_from_extern[0] -> Discard;
	my_ip_from_extern[1] -> [1]rw; 
	my_ip_from_extern[2] -> Discard;
	my_ip_from_extern[3] -> Discard;"

"ip_from_extern[1] -> Discard;"


// FILTER & REWRITE IP PACKETS FROM INSIDE

"ip_from_intern :: IPClassifier(dst host intern,
			dst net intern,
			-);"

"intern_arp_class[2] -> Strip(14)
  	-> CheckIPHeader()
	-> ip_from_intern;
ip_from_intern[0] -> Discard; 
ip_from_intern[1] -> Discard;
ip_from_intern[2] -> [0]rw;";


