/*
 * peer.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef PEER_HEADER
#define PEER_HEADER

#include <nodeos/an_chan.h>
#include <nodeos/an_cond.h>

#include "types.h"
#include "tic.h"
#include "ticlist.h"
#include "debug.h"

#define BUFS_PER_PEER  8

typedef struct Peer {
	int index;
	Uint64 identity;
	Boolean joined;

	const char* name;
	/* 
	 * Set if this node needs to request the peer
	 * retransmit some packets.
	 */
	Boolean needRetransmit;
	/*
	 * Highest number tic received from this Peer.
	 */
	Tic madeToTic;
	/*
	 * Highest number tic sent to this Peer.
	 */
	Tic lastSentTic;			DEBUGONLY(Uint32 magic1);
	/*
	 * Tic of recv'd quit command, or NOT_QUIT if none recv'd
	 */
	Tic quitTic;
	/*
	 * Buffer of tic commands from remote players.  Valid between
	 * (gameState.playedToTic%TIC_HISTORY_LEN) and
	 * (<this>.madeToTic%TIC_HISTORY_LEN).
	 */
	TicList ticCommand;
	/* TicCommand ticCommand[TIC_HISTORY_LEN]; */
	DEBUGONLY(Uint32 magic2);
	an_chan_t sendChan;			DEBUGONLY(Uint32 magic3);
	an_chan_t recvChan;			DEBUGONLY(Uint32 magic4);
	an_demuxkey_t recvDemux;		DEBUGONLY(Uint32 magic5);
	// actual recv buffers: XXX rename to recvBufs
	an_pbuf_t bufs[BUFS_PER_PEER];		DEBUGONLY(Uint32 magic6);
	// pointers to buffers: XXX rename to recvBufPtrs
	an_pbuf_t *bufptrs[BUFS_PER_PEER];	DEBUGONLY(Uint32 magic7);

	struct Peer* next;
} Peer;

#define NOT_QUIT	(Uint64)(-1)

#define PEER_MAGIC_1 0x11111111
#define PEER_MAGIC_2 0x22222222
#define PEER_MAGIC_3 0x33333333
#define PEER_MAGIC_4 0x44444444
#define PEER_MAGIC_5 0x55555555
#define PEER_MAGIC_6 0x66666666
#define PEER_MAGIC_7 0x77777777

extern Peer *peerList;

extern Peer* createPeer(const char *peerAddr, 
			const char *myAddr, 
			const char *protocol,
			Boolean createRealPeer);
extern const char *peertostring(Peer* p, char *buf, int buflen);
extern void strtoaddrport(const char *id, Uint32 *addr, Uint16 *port);
extern int peerCount(void);
extern void checkPeerMagic(Peer *peer);

#endif /* PEER_HEADER */
