/*
 * server.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef SERVER_HEADER
#define SERVER_HEADER

#include <nodeos/an_chan.h>

#include "tic.h"

#define BUFS_PER_SERVER 64

/*
 * XXX there is a lot of overlap between Server and Peer.
 */
typedef struct Server {
	const char* name;
	/* 
	 * Set if this node needs to request the server
	 * retransmit some packets.
	 */
	Boolean needRetransmit;
	/*
	 * Highest number tic received from this Server.
	 */
	Tic madeToTic;
	/*
	 * Highest number tic sent to this Server.
	 */
	Tic lastSentTic;

	/*
	 * XXX no cache of packets recv'd from the server (like for Peers)
	 * currently we push the server state out into local
	 * peer state.  That's pretty bogus, though.
	 */

	an_chan_t sendChan;
	an_chan_t recvChan;
	an_demuxkey_t recvDemux;
	// actual recv buffers:
	an_pbuf_t recvBufs[BUFS_PER_SERVER];
	// pointers to buffers:
	an_pbuf_t *recvBufPtrs[BUFS_PER_SERVER];

	struct Server* next;
} Server;

Server* createADoomServer(const char *serverAddr, 
			  const char *myAddr,
			  const char *protocol);

Server* adoomServer(void);

#endif /* SERVER_HEADER */
