/*
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Test the fs.  
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <pthread.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_fs.h>

static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };

int
ee_main(int argc, const char **argv)
{
	const char *filename = 0;
	an_file_t *file;
	an_error_t error;
	an_acd_t acd;

#if defined(OSKIT) && defined(GPROF)
	oskit_start_gprof();
#endif

	/* try creating a file */
	printf("Create test.\n");
	acd.st_mode = 0644;
	filename = "./mytestfile";
	file = 0;
	error = an_file_open(filename, AN_O_CREAT | AN_O_RDWR, 
			     &acd, &file);
	if (error == 0) {
		static const char *msg = "Hello World!";
		int bytes_out;

		/* Quick check of reading/writing */
		if( !an_file_write(file, strlen(msg) + 1,
				   (void *)msg, &bytes_out) ) {
			const char *msg_ptr = msg;
			int bytes_in, done = 0;
			an_offset_t off;
			an_stat_t an_st;
			char b;

			assert(bytes_out == (strlen(msg) + 1));
			error = an_file_fstat(file, &an_st);
			assert(error == 0);
			assert(an_st.st_size == bytes_out);
			error = an_file_lseek(file, 0, SEEK_SET, &off);
			while( !done || error ) {
				if( (error = an_file_read(file,
							  1,
							  &b,
							  &bytes_in)) == 0 ) {
					assert(bytes_in == 1);
					assert(b == *msg_ptr);
					if( *msg_ptr == 0 )
						done = 1;
					else
						msg_ptr++;
				}
			}
			if( !error ) {
				error = an_file_read(file, 1, &b, &bytes_in);
				assert(error == 0);
				assert(bytes_in == 0);
			}
		} else {
			printf("Error: Couldn't write to %s\n", filename);
		}
	        error = an_file_close(file);
		assert(error == 0);
		error = an_file_unlink(filename);
		assert(error == 0);
	} else {
		printf("Warning: Couldn't create %s\n", filename);
	}

	/* try opening a non-existent file */
	printf("Open non-existent file.\n");
	file = 0;
	error = an_file_open("./thisdoesnotexist", AN_O_RDWR, &acd, &file);
	assert (error != 0); /* XXX error == AN_E_FILENOTFOUND(?) */

	/* test writing a file, in chunks */
	/* XXX */

	/* test overwriting part of a file */
	/* XXX */

	/* test seek */
	/* XXX */

	/* open same file multiple times */
	/* XXX */

	/* try a redundant close */
	an_file_close(file);
	an_file_close(file);

#if defined(OSKIT) && defined(GPROF)
	oskit_stop_gprof();
#endif
	printf("Done.\n");

	return 0;
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "fs" }; 
	an_domain_t *domain = an_domain_current();

	status = ee_main(argc, argv);
	an_domain_destroy(domain);
}
