/*
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * EE to report system info
 */
#include <stdio.h>
#include <nodeos/an_impl.h>
#include <nodeos/an_info.h>

static char *mactostr(unsigned char mac[6]);
static char *iptostr(u_int32_t addr);
static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };

int
ee_main(int argc, const char **argv)
{
	an_error_t err;
	an_infocpu_t cpu;
	an_infoif_t infoif[NODEOS_MAXIFS];
	an_infoipv4_t infoipv4[NODEOS_MAXIFS];
	int nif, i, noipv4 = 0;

	err = an_info_getcpu(&cpu);
	if (err != 0)
		printf("WARNING: no CPU info\n");
	else
		printf("%d CPUs, %u cycles per second\n",
		       cpu.ncpu, cpu.cps);

	/*
	 * Get interface info
	 */
	nif = NODEOS_MAXIFS;
	err = an_info_getifs(infoif, &nif);
	if (err != 0)
		printf("WARNING: no interface info\n");
	else {
		/*
		 * Get IPv4 info if available
		 */
		i = NODEOS_MAXIFS;
		err = an_info_getipv4(infoipv4, &i);
		if (err != 0 || i == 0) {
			printf("WARNING: no interface IPv4 info\n");
			noipv4 = 1;
		}

		printf("%d network interface(s):\n", nif);
		for (i = 0; i < nif; i++) {
			printf("  %d: %s (%s): %u Mbps",
			       infoif[i].id, infoif[i].pname,
			       mactostr(infoif[i].addr.eth.daddr),
			       infoif[i].bandwidth);
			if (!noipv4) {
				assert(infoif[i].id == infoipv4[i].ifid);
				printf(": IPaddr=%s",
				       iptostr(infoipv4[i].addr));
				printf(", IPmask=%s",
				       iptostr(infoipv4[i].mask));
			}
			printf("\n");
		}
	}

	return 0;
}

static char *
mactostr(unsigned char mac[6])
{
	static char macstr[6*3+1];

	snprintf(macstr, sizeof macstr, "%02x:%02x:%02x:%02x:%02x:%02x",
		 mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	return macstr;
}

static char *
iptostr(u_int32_t addr)
{
	static char ipstr[4*4+1];
	union addru {
		u_int32_t i;
		u_int8_t o[4];
	} *addru = (union addru *)&addr;

	snprintf(ipstr, sizeof ipstr, "%u.%u.%u.%u",
		 addru->o[0], addru->o[1], addru->o[2], addru->o[3]);
	return ipstr;
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "ChantestEE" }; 

	status = ee_main(argc, argv);
	an_domain_destroy(an_domain_current());
}
