/*
 * conglom.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * These are general helpers for conglomeration and deconglomeration logic
 */

/*
 * Maximum number of bytes to stick in a single conglom packet
 */
#define MAX_CONGLOM_SIZE	1024

/*
 * Performs the math to find the maximum size to allocate a buffer that
 * contains as many packets of pktsize as possible without going over
 * MAX_CONGLOM_SIZE, or, if count packets can already fit, then returns
 * the size to allocate for count packets
 */
static inline int
getConglomBufSize(int pktsize, int count)
{
	int size;

	size = pktsize+ticCommandSize()*count;
	if (size > MAX_CONGLOM_SIZE)
	{
		/* Kinky */
		size = pktsize +
			(int)((MAX_CONGLOM_SIZE - pktsize)/ticCommandSize()) *
			ticCommandSize();
	}
	assert(size >= pktsize+ticCommandSize());
	assert(size <= MAX_CONGLOM_SIZE);
	
	return size;
}

