/*
 * identity.c
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <string.h>

#include "identity.h"
#include "error.h"
#include "trace.h"
#include "network.h"

Uint64
computeIdentity(const char *protocol, const char* name)
{
	Uint64 identity;

	if (!strcasecmp(protocol, "udp"))
	{
		Uint32 addr;
		Uint16 port;
		strtoaddrport(name, &addr, &port);
		identity = (((Uint64)(addr)) << 16)
			| (Uint64)(port);
	}
	else
	{
		FATALERROR("Dunno how to make an identity for protocol `%s'\n",
			   protocol);
	}

	TRACE(TRACE_SETUP, "identity for `%s' is %#qx\n", name, identity);

	return identity;
}

