/*
 * options.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef OPTIONS_HEADER
#define OPTIONS_HEADER

#include <nodeos/an_domain.h>
#include "types.h"

typedef enum OptionType {
	OT_INT=11, OT_BOOL=12, OT_STR=13, OT_INETADDR=14
} OptionType;

typedef struct OptionSpec {
	const char *name;
	OptionType type;
	void *addr;
	union {
		struct {
			int defaultValue;
			int min;
			int max;
		} intspec;
		struct {
			Boolean defaultValue;
		} boolspec;
		struct {
			const char* defaultValue;
		} strspec;
		struct {
			const char* defaultValue;
		} inetaddrspec;
	} spec;
} OptionSpec;

extern int parseArgs(an_initargs_t* args, OptionSpec *argSpecs, int specSize);

#endif /* OPTIONS_HEADER */
