/*
 * tic.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef TIC_HEADER
#define TIC_HEADER

#include "types.h"
#include "debug.h"

#define TIC_HISTORY_LEN 16

/*
 * Cap the gap between created tics and played tics to
 * one-half the length of the tic buffer.
 */
#define MAX_TICS_AHEAD ((TIC_HISTORY_LEN/2)-1)

typedef Uint64 Tic;

typedef struct TicCommand {
	DEBUGONLY(Uint64 identity;
		  Tic tic;
		  int spiff;       /* Give this structure a life.  This is
				    * *completely* arbitrary and is only here
				    * for debugging */
		);
	unsigned char flags;       /* This would presumably be used by the tic
				    * command structure itself.  Currently it
				    * get set to 0x1 to indicate a disconnect
				    * packet */
	int fillOutTheStruct[0];
} TicCommand;

#define TFLAG_QUIT	0x01

/* The following is not actually declared in libadoom, it is declared in main.c
 * of both fdoom and adoom
 */
extern int ticfillsize;

#define ticCommandSize() (sizeof(TicCommand)+ticfillsize)

extern void makeTic(Tic tic, TicCommand* ticCmd, Uint64 identity);

#endif /* TIC_HEADER */
