/*
 * ticlist.c
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <nodeos/an_debug.h>

#include <assert.h>  // assert
#include <stdlib.h>  // malloc, free
#include <string.h>  // memcpy

#include "ticlist.h"

void
ticListInit(TicList *tl)
{
	int i;
	
	tl->size = TIC_HISTORY_LEN;

	/* Allocate a continguous memory chunk for tics */
	tl->ticChunk = malloc(tl->size*ticCommandSize());
	assert(tl->ticChunk);

	for (i = 0; i < tl->size; i++)
	{
		tl->tics[i] = tl->ticChunk+i*ticCommandSize();
	}

	DEBUGONLY(
		for (i = 0; i < TIC_HISTORY_LEN; i++) tl->whichTics[i]=(Uint64)(-1);
		);
}

void
ticListDestory(TicList *tl)
{
	/* Free chunk (NOT individual tics) */
	if (tl->ticChunk)
		free(tl->ticChunk);
	tl->ticChunk = NULL;

	tl->size = 0;
}

void
ticListCopyTicOut(TicList *tl, Tic tic, TicCommand *dest)
{
	memcpy(dest, ticListGetTicForGet(tl, tic), ticCommandSize());
}

void
ticListInsertOneTic(TicList *tl, Tic tic, TicCommand *src)
{
	memcpy(ticListGetTicForPut(tl, tic), src, ticCommandSize());
}

/*
 * The following two functions differ only in debug code.
 */
/*
 * ticListGetTicForPut is meant to be used when the pointer is not
 * intended to be used for reading information out of the tic, but
 * only for putting information into it.
 */
TicCommand *
ticListGetTicForPut(TicList *tl, Tic tic)
{
	DEBUGONLY(tl->whichTics[tic % tl->size] = tic);
	return tl->tics[tic % tl->size];
}

/*
 * ticListGetTicForGet is meant to be used when the pointer is
 * intended to be used for reading information out of the tic, and
 * not for putting information info it.
 */
TicCommand *
ticListGetTicForGet(TicList *tl, Tic tic)
{
	DEBUGONLY(assert(tl->whichTics[tic % tl->size] == tic));
	return tl->tics[tic % tl->size];
}

void
ticListInsertRunningPacket(TicList *tl, RunningPacket *packet)
{
	Tic tic;
	int i;
	
	for (tic = packet->startTic, i = 0; i < packet->ticCount; tic++, i++)
	{
		DEBUGONLY(assert(packet->ticCommand[i].tic == tic));
		ticListInsertOneTic(tl, tic, &packet->ticCommand[i]);
	}
}

