/*
 * trace.h
 *
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 *
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef TRACE_HEADER
#define TRACE_HEADER

#include "types.h"

#define SET_BIT(x)		(TraceMask)((1<<(x)))
#define TRACE_SETUP		SET_BIT(1)
#define TRACE_SEND		SET_BIT(2)
#define TRACE_SEND_DELAY	SET_BIT(3)
#define TRACE_SEND_ERR		SET_BIT(4)
#define TRACE_RECV		SET_BIT(5)
#define TRACE_TIC		SET_BIT(6)
#define TRACE_RETRAN		SET_BIT(7)
#define TRACE_GSLOCK		SET_BIT(8)
#define TRACE_CONGLOM		SET_BIT(9)
#define TRACE_KICK		SET_BIT(10)
#define TRACE_NONE		((TraceMask)(0))
#define TRACE_ALL		((TraceMask)(-1))

typedef Uint64 TraceMask;

extern TraceMask traceMask;

extern const char* traceInit(const char *strmask);

extern void trace(const char* func, int line, TraceMask mask, const char *fmt, ...)
#ifdef __GNUC__
	__attribute__((format (printf, 4, 5)))
#endif
;

#if defined(DEBUG) && !defined(NDEBUG)
#define TRACEONLY(x) x
#define TRACE(mask, fmt, args...) trace(__FUNCTION__, __LINE__, mask, fmt, ## args)
#else
#define TRACEONLY(x)
#define TRACE(mask, fmt, args...)
#endif

#endif /* TRACE_HEADER */
