/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#undef VERBOSE

/*
 * Reply to UDP ping packets (shamelessly stolen from OSKit pingreply).
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <nodeos/an_impl.h>
#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_chan.h>
#include <nodeos/an_mutex.h>
#include <nodeos/an_pbuf.h>
#include <nodeos/an_info.h>
#include <nodeos/an_event.h>
#include <nodeos/lib/arplib.h>
#include <nodeos/lib/pbcache.h>
#include <nodeos/lib/utils.h>

#include <nodeos/impl/an_gprof.h>	/* XXX */

#include "timeout.h"

#define RUNFOR		60	/* run for this many seconds */
#ifdef arm32
#define HOSTADDR	155,99,214,44	/* dnard184 */
#define IFNUM		0		/* interface to use; i.e. "eth<IFNUM>" */
#else
//#define HOSTADDR	206,163,153,26  /* Poohbear */
//#define HOSTADDR	155,99,214,151	/* chukar */
//#define HOSTADDR	155,99,214,138	/* tbpc12 */
//#define HOSTADDR	155,99,214,133	/* tbpc33 */
//#define HOSTADDR	192,168,1,1	/* tbpc33 -- jsac vlan */
#define HOSTADDR	192,168,3,2	/* tbpc03:0 */
//#define IFNUM		4		/* interface to use; i.e. "eth<IFNUM>" */
#define IFNUM		0		/* interface to use; i.e. "eth<IFNUM>" */
#endif
#define MAXTHREADS	2	/* max threads handling packets at once */
#define MAXBUFS		32	/* max packet buffers for those threads */

static unsigned char myip[4] = { HOSTADDR };

/*
 * For some reason the headers define ETHER_MAX_LEN as
 * (ETH_FRAME_LEN + ETH_CRC_LEN), but then don't define ETH_CRC_LEN.
 */
#ifdef __linux__
#ifndef ETH_CRC_LEN
#define ETH_CRC_LEN ETHER_CRC_LEN
#endif
#endif

/*
 * XXX hack arp tab cuz I don't want to do the protocol
 */
struct arpentry {
	unsigned char ethaddr[6];
	unsigned char ipaddr[4];
	an_chan_t *chan;
	an_chan_t chan_data;
};
#define MAXARPTAB 6
static struct arpentry arptab[MAXARPTAB];

static pbcache_t *pbufcache;

static void ee_init(void *arg);
an_domaininit_func_t InitEE = { ee_init };

static char *iptoa(unsigned long x);
static void ee_die(an_event_t *event, void *arg);
static void ee_ip(an_pbuf_t *pbuf, void *arg, int msize);
static void setup_channels(void);

static an_demuxkey_t ipdemux_data, *ipdemux;
static an_chan_t *ipinchan, ipinchan_data, *ipoutchan;
static unsigned int myipaddr;
static int dying;
static char ifname[16];

an_ethaddrspec_t myaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};
an_ethaddrspec_t ipoutaddr = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, {0}, ETHERTYPE_IP
};

int
ee_main(int argc, const char **argv)
{
	assert(argc >= 1);

	printf("%p/%p: %s started\n",
	       an_domain_current(), an_thread_current(), argv[0]);
	memcpy(&myipaddr, myip, sizeof myipaddr);

	/*
	 * Initialize Arp library and configure our interface,
	 */
	arp_init();
	arp_config_iface(IFNUM, (struct in_addr *) &myipaddr);

	/*
	 * Setup channels
	 */
	pbufcache = pbcache_create(MAXBUFS, 0);
	setup_channels();

	printf("%s (%02x:%02x:%02x:%02x:%02x:%02x) ready\n",
	       iptoa(myipaddr),
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5]);

	an_gprof_start(argv[0]);
	return 0;
}

/* The address we are passed is in network order. */
static char *
iptoa(unsigned long x)
{
	static char buf[4*4];		/* nnn.nnn.nnn.nnnz */
	unsigned char *p = (unsigned char *)&x;

	sprintf(buf, "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
	return buf;
}

static an_chan_t *
find_channel(struct ip *ip)
{
	static struct arpentry *lastent = arptab;
	struct arpentry *aent, *freeent = 0;

	/*
	 * Find the table entry
	 */
	aent = lastent;
	do {
		if (memcmp(aent->ipaddr, &ip->ip_src.s_addr, 4) == 0) {
			lastent = aent;
			return aent->chan;
		}
		if (!freeent && !aent->chan)
			freeent = aent;
			
		if (++aent == &arptab[MAXARPTAB])
			aent = arptab;
	} while (aent != lastent);

	if (!freeent)
		return 0;

	{
		struct {
			an_chanaddr_t addr;
			an_chanproto_t proto;
		} attr;
		an_ethaddrspec_t fooaddr;
		int		 err;
		
		memset(&fooaddr, 0, sizeof(fooaddr));
		err = arp_arpfor(IFNUM, &ip->ip_src, fooaddr.daddr);
		if (err)
			return 0;

		memset(&attr, 0, sizeof attr);
		strcpy(attr.proto.data, ifname);
		attr.proto.len = strlen(attr.proto.data);
		memcpy(attr.addr.eth.daddr,
		       fooaddr.daddr, sizeof(fooaddr.daddr));
		attr.addr.eth.type = htons(ETHERTYPE_IP);

		freeent->chan = an_outchan_create(&freeent->chan_data,
						  an_domain_current(),
						  &attr.proto, &attr.addr, 0);
		assert(freeent->chan != 0);
		memcpy(freeent->ipaddr, &ip->ip_src.s_addr, 4);
#ifdef VERBOSE
		printf("created output channel %d for %s\n",
		       freeent-arptab, iptoa(ip->ip_src.s_addr));
#endif
	}
	return freeent->chan;
}

static void
setup_channels(void)
{
	an_error_t err;
	struct {
		an_chanaddr_t addr;
		an_chanproto_t proto;
		an_netspec_t net;
		an_pbuf_t *bufs[MAXBUFS];
	} attr;
	an_domain_t *domain;
	an_infoif_t info[8];
	int nif = 8;
	int i;

	/*
	 * Find my address.
	 * XXX assumes first interface is ethernet.
	 */
	err = an_info_getifs(info, &nif);
	assert(err == 0);
	assert(nif >= IFNUM);
	assert(info[IFNUM].type == ANIF_ETH);
	memcpy(myaddr.daddr, info[IFNUM].addr.eth.daddr, sizeof myaddr.daddr);
	sprintf(ifname, "if%d", info[IFNUM].id);

	/*
	 * Setup an event to time us out in RUNFOR seconds
	 */
	set_die_timeout(RUNFOR, ee_die);
	printf("Timer started, running for %d seconds\n", RUNFOR);
	
	domain = an_domain_current();
	memset(&attr, 0, sizeof attr);

	/*
	 * Setup in channel for ICMP/IP packets
	 * Again, we first build up the demux.  Note that due to the
	 * potentially variable-length IP header, we cannot check the ICMP
	 * type.  So all we can demux on is the IP type and even that might
	 * not be reliable since we cannot verify the checksum.
	 */
	ipdemux = an_demuxkey_create(&ipdemux_data, domain);
	assert(ipdemux != 0);
	err = an_demuxkey_addipv4(ipdemux,
				  (AN_DEMUXKEY_IPV4_ANYSADDR|
				   AN_DEMUXKEY_IPV4_ANYDADDR),
				  0, 0, 0, 0, IPPROTO_ICMP);

	strcpy(attr.proto.data, ifname);
	attr.proto.len = strlen(attr.proto.data);
	myaddr.type = htons(myaddr.type);
	attr.addr.eth = myaddr;
	for (i = 0; i < MAXTHREADS; i++)
		attr.bufs[i] = pbcache_alloc(pbufcache, 0);
	attr.net.maxthreads = MAXTHREADS;
	attr.net.buffers.npbufs = MAXTHREADS;
	attr.net.buffers.pbufs = attr.bufs;
	ipinchan = an_inchan_create(&ipinchan_data, domain, ipdemux,
				    &attr.proto, &attr.addr, &attr.net,
				    ee_ip, 0);
	assert(ipinchan != 0);

	printf("%p: inchannel for %02x:%02x:%02x:%02x:%02x:%02x, type=%x\n",
	       ipinchan,
	       myaddr.daddr[0], myaddr.daddr[1], myaddr.daddr[2],
	       myaddr.daddr[3], myaddr.daddr[4], myaddr.daddr[5],
	       ntohs(myaddr.type));
}

static void
ee_ip(an_pbuf_t *pbuf, void *arg, int msize)
{
	int psize = an_pbuf_size(pbuf);
	void *data = an_pbuf_data(pbuf);
	int ipdata[ETHER_MAX_LEN/sizeof(int)+1];
	static int ipid = 0xb;		/* something random */
	struct ip *ip;
	int hlen;
	struct icmp *icmp;
	an_chan_t *outchan;
	an_error_t err;

	assert(msize >= sizeof *ip);

	if (dying)
		goto done;

	an_inchan_addpbuf(ipinchan, pbcache_alloc(pbufcache, 0));

	if (psize < sizeof *ip) {
#ifdef VERBOSE
		printf("yow! IP packet too small (%d < %d)\n",
		       psize, sizeof *ip);
#endif
		goto done;
	}

	/* XXX deal with fragments. */

	ip = (struct ip *)data;
	hlen = ip->ip_hl << 2;
	assert(hlen == msize);

	/*
	 * Copy out from ip header on to ensure proper alignment
	 * on machines that are not byte addressable!
	 */
	memcpy(ipdata, data, ntohs(ip->ip_len));
	ip = (struct ip *) ipdata;

	assert(ip->ip_p == IPPROTO_ICMP);
	if (ipcksum(ip, hlen) != 0)
		goto done;		/* bad cksum */

	icmp = (struct icmp *)((char *)ip + hlen);
	if (icmp->icmp_type != ICMP_ECHO
	    || ipcksum(icmp, ntohs(ip->ip_len) - hlen) != 0)
		goto done;		/* bad cksum or not ICMP_ECHO */

	/*
	 * Find/create a channel to send on
	 */
	outchan = find_channel(ip);
	if (outchan == 0) {
#ifdef VERBOSE
		printf("yow! I do not know about %s\n",
		       iptoa(ip->ip_src.s_addr));
#endif
		goto done;
	}

	/* Send the reply. */
	icmp->icmp_type = ICMP_ECHOREPLY;
	/* Another optimization.  Only the icmp type field in the reply
	 * changes, so we can use this apriori knowledge to determine
	 * the new checksum based upon the old checksum.  Eliminates
	 * touching all of the data in the packet once more */
#if 0
	icmp->icmp_cksum = 0;
	icmp->icmp_cksum = ipcksum(icmp, ntohs(ip->ip_len) - hlen);
#else
	if (icmp->icmp_cksum >= 65527)
		icmp->icmp_cksum += (unsigned short) 9;
	else
		icmp->icmp_cksum += (unsigned short) 8;
#endif

	ip->ip_id = htons(ipid++);

	/*
	 * We can't just swap the addresses,
	 * since broadcast packets didn't come to our address.
	 */
	ip->ip_dst = ip->ip_src;
	ip->ip_src.s_addr = myipaddr;

	ip->ip_sum = 0;
	ip->ip_sum = ipcksum(ip, hlen);

	/*
	 * Copy data back into original data buffer.
	 */
	memcpy(data, ip, ntohs(ip->ip_len));

#ifdef VERBOSE
	printf("sending ICMP reply to %s on %p\n",
	       iptoa(ip->ip_dst.s_addr), outchan);
#endif

	err = an_outchan_send(outchan, pbuf);
	assert(err == 0);
	ipoutchan = outchan;

done:
	pbcache_free(pbufcache, pbuf);
}

static void
ee_die(an_event_t *event, void *arg)
{
	dying = 1;
	an_gprof_stop();

	moab_chan_printstats(ipinchan, 1, "Request", RUNFOR, 0);
	moab_chan_printstats(ipoutchan, 1, "Reply", RUNFOR, 0);

	if (ipinchan)
		an_inchan_destroy(ipinchan);
	arp_exit();
	pbcache_destroy(pbufcache, 0);
	an_domain_destroy(an_domain_current());
}

static void
ee_init(void *arg)
{
	int status;
	int argc = 1;
	const char *argv[] = { "PingreplyEE" }; 

	status = ee_main(argc, argv);
}
