/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
/*
 * rootdomain.c --- test parts of the root domain.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h> /* XXX memset */

#include <nodeos/an_domain.h>
#include <nodeos/an_thread.h>
#include <nodeos/an_moab_setup.h>
#include <nodeos/an_debug.h>

static void rootdomain_threadfunc(void *arg);

static an_cred_t root_cred;
static an_mempool_t root_mempool;
static an_domain_t root_domain;

#define ROOTDOMAIN_THREAD_STACKSIZE  (20 * 1024) /* OSKit pthread minimum */

typedef struct { char stack[ROOTDOMAIN_THREAD_STACKSIZE];} stackhack_t;

static an_thread_t prealloced_threads[5];
static stackhack_t prealloced_stacks[5];

/*
 * Allocates N threads in root domain (N defaults to 1).
 */
int
main(int argc, char **argv)
{
	void *threads[5];
	void *stacks[5];
	int threadct = 1;
	int i;

	if (argc > 1 && strcmp(argv[1], "-moabdebug") == 0) {
		an_debug_setmask(argv[2]);
		argc -=2, argv += 2;
	}

	if (argc > 1)
		threadct = atoi(argv[1]);

	assert(threadct <= 5);
	assert(threadct > 0);

	for (i = 0; i < threadct; i++) {
		threads[i] = &prealloced_threads[i];
		stacks[i] = &prealloced_stacks[i];
		memset(stacks[i], 0, ROOTDOMAIN_THREAD_STACKSIZE);
	}

	/*
	 * This code all runs without a domain or NodeOS thread
	 * context.  There are few things that can be done
	 * in this context.
	 */
	an_moab_setup();
	
	printf("Utah NodeOS (version 0.%04d)\n", an_moab_version());

	an_moab_start(&root_domain, &root_cred, &root_mempool,
		      threadct, ROOTDOMAIN_THREAD_STACKSIZE,
		      threads, stacks,
		      rootdomain_threadfunc, NULL);
	
	printf("rootdomain: PASS\n");
	printf("Root domain died, exiting...\n");

	return 0;
}

static void
rootdomain_threadfunc(void *arg)
{
	printf("First (and only) thread in root domain started.\n");

	an_moab_shutdown();
}
