/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef __ANLIB_CHAN_IN_H__
#define __ANLIB_CHAN_IN_H__

/* Protocol types */
typedef enum {
	CHANPROTO_NONE, CHANPROTO_INVALID,
	CHANPROTO_IN_ETH, CHANPROTO_IN_IF, CHANPROTO_IN_IP, CHANPROTO_IN_UDP,
	CHANPROTO_IN_ARP, 
	CHANPROTO_OUT_ETH, CHANPROTO_OUT_IF, CHANPROTO_OUT_IP, CHANPROTO_OUT_UDP, 
	CHANPROTO_OUT_UDPIP, CHANPROTO_OUT_ARP
} anlib_chanproto_t;

struct pcg;
union anlib_chanaddr;

typedef struct addr_data {
        int len;
        char *cur;
} addr_data_t;

typedef struct pcg_ops {
	anlib_chanproto_t id;
	int (*matches)(char *proto, int len, char *str);
	char *matches_arg;
	int (*update_demuxkey)(addr_data_t *addr, int offset, 
					an_demuxkey_t *key);
	int (*create_graph)(union anlib_chanaddr *addr, an_mempool_t *mpool,
							anlib_clickgraph_t **pg, char *config);
	char *create_arg;
	an_error_t (*destroy)(anlib_clickgraph_t *graph);
} pcg_ops_t;

typedef struct pcg {
	pcg_ops_t *ops;
	anlib_clickgraph_t *graph;
	struct pcg *next;
} pcg_t;

int validate_proto(an_chanproto_t *p);
anlib_chanproto_t get_next_proto(an_chanproto_t *proto, int chantype, 
		an_mempool_t *mpool, pcg_t **pcg, pcg_t **ppcg_head, void **last);
int get_next_address(addr_data_t *addr, void *out_addr, int proto, 
		int *flags);
int connect_pcgs(pcg_t *pcg_head);
int connect_pcgpair(pcg_t *p1, pcg_t *p2);
pcg_t *
alloc_pcg(pcg_t **ppcg_head, an_mempool_t *mpool);


int get_component_len(char **pstr, int *plen);
int anlib_chan_cleanpcgs(pcg_t *pcg, an_mempool_t *mp);

int anlib_chanprefix_matches(char *proto, int len, char *notused);
int anlib_chan_matches(char *proto, int len, char *str);
int anlib_ipUpdateDemux(addr_data_t *addr, int offset, an_demuxkey_t
		*key);
int anlib_udpUpdateDemux(addr_data_t *addr, int offset, an_demuxkey_t *key);
int anlib_udpipUpdateDemux(anlib_chanaddr_t *addr, int offset, an_demuxkey_t *key, void
		**last);
int anlib_inchan_createGraph(anlib_chanaddr_t *addr, an_mempool_t *mpool,
		anlib_clickgraph_t **pg, char *config);

int 
anlib_outchan_arpDestroy(anlib_clickgraph_t *graph);
int
anlib_outchan_arpCreateGraph(an_domain_t *domain, anlib_chanaddr_t *addr, 
				pcg_t *pcg, an_chanproto_t *native_proto, an_netspec_t *net);
#endif /* __ANLIB_CHAN_IN_H__ */
