/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Click graphs (configurations) for individual protocol components that 
 * are used to create channels.
 */

/* 
 * Unsafe elements
 * 1. CheckIPHeader - Already safe by using u_atomic32_t for drop count
 * 2. IPReassembler - Done. Added a ReadWriteLock.
 * 3. CheckUDPHeader - uses u_atomic32_t
 * 4. CheckIP6Header - Only state is _drops, make it u_atomic32_t, if we care
 * 5. ARPResponder (No need to make it safe as of now, coz changes state only
 *     in live_reconfigure)
 */
 
/* Input channel configs */

/* IP packets received would have their IP header intact */
char anlib_inchan_ipConfig[] = 
	"out :: ToOutChan(out);
	in :: FromInChan(in);
	cip :: CheckIPHeader;
	ipr :: IPReassembler;
	in -> cip -> ipr -> out;";

/* The user gets to see only the pay-load carried in the packet */
/* Issue: What if the user doesn't specify an address and wants to see the
 * headers?? The work-around for now would be create a different channels and
 * specify the protocol selection in the demuxkey.
 */
char anlib_inchan_udpConfig[] = 	
	"in :: FromInChan(in);
	out :: ToOutChan(out);
	cudp :: CheckUDPHeader;
	in -> MarkIPHeader() -> cudp -> StripIPHeader() -> Strip(8) -> out;";

char anlib_inchan_ip6Config[] =
	"in :: FromInChan(in);
	out :: ToOutChan(out);
	cip :: CheckIP6Header;
	in -> cip -> out;";

char anlib_inchan_arpConfig[] = 
	"in :: FromInChan(in);
	out :: ToOutChan(out);
	ar :: ARPResponder(%d.%d.%d.%d %02x:%02x:%02x:%02x:%02x:%02x);
	in -> ar -> out;";
	
/* Output channel configs */

char anlib_outchan_ipConfig[] = 
	"in :: FromInChan(in);
	out :: ToOutChan(out);
	ipencap :: IPEncap(%u, %d.%d.%d.%d, %d.%d.%d.%d); 
	in -> ipencap -> out;";

char anlib_outchan_udpipConfig[] =
	"in :: FromInChan (in);
	out :: ToOutChan (out);
	en :: UDPIPEncap (%d.%d.%d.%d, %u, %d.%d.%d.%d, %u); 
	in -> en -> out;";

char anlib_outchan_udpConfig[] = 
	"in :: FromInChan (in);
	out :: ToOutChan (out);
	en :: UDPIPEncap (0.0.0.0, %u, 0.0.0.0, %u); 
	delip :: StripIPHeader();
	in -> en -> delip -> out;";

/* ARP channel */
char anlib_outchan_arpConfig[] =
	"in0 :: FromInChan(in);
	in1 :: FromInChan(arpr);
	out :: ToOutChan(out);
	arpq :: ARPQuerier(%d.%d.%d.%d, %02x:%02x:%02x:%02x:%02x:%02x);
	in0 -> MarkIPHeader -> [0] arpq -> out; 
	in1 -> [1] arpq;";

/* Miscellaneous */

char anlib_strip_ethConfig[] = 
    "FromInChan(in) -> Strip(14) -> ToOutChan(out);";
	
char anlib_printConfig[] = 
    "FromInChan(in) -> Print(%s) -> ToOutChan(out);";

/* End-of-file */
