/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Read and parse a simple config file format.  Format is:
 *	# comment
 *	<param string name>	<param value>
 *	...
 * where param value is either an int or string as determined by the
 * configuration description.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <nodeos/lib/configfile.h>

/*
 * XXX convert to use an_file interface
 */
int
config_parse(const char *cfile, config_param_t cparams[], int numparams)
{
	FILE	*fp;
	int	i;
	char	buf[BUFSIZ], *bp, variable[256], value[256];

	if ((fp = fopen(cfile, "r")) != NULL) {
		while (1) {
			if ((bp = fgets(buf, BUFSIZ, fp)) == NULL)
				break;

			if (*bp == '\n' || *bp == '#')
				continue;

			sscanf(bp, "%s %s\n", variable, value);

			for (i = 0; i < numparams; i++) {
				if (! strcmp(cparams[i].name, variable))
					break;
			}
			if (i == numparams)
				continue;

			switch (cparams[i].type) {
			case CONFIG_STRING:
				bp = malloc(strlen(value) + 1);
				assert(bp);
				strcpy(bp, value);
				cparams[i].value.s = bp;
				break;
			case CONFIG_INT:
			        cparams[i].value.i = atoi(value);
				break;
			}
		}
		fclose(fp);
	}

	return 0;
}

void
config_dump(const char *str, config_param_t cparams[], int numparams)
{
	int i;

	printf("%s configuration:\n", str);
	for (i = 0; i < numparams; i++) {
		printf("  %s ", cparams[i].name);
		switch (cparams[i].type) {
		case CONFIG_STRING:
			printf("%s\n", cparams[i].value.s ?
			       cparams[i].value.s : "<unset>");
			break;
		case CONFIG_INT:
			printf("%d\n", cparams[i].value.i);
			break;
		}
	}
}
