/*
 * Copyright (c) 1999-2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Domain API
 */
#ifndef _NODEOS_DOMAIN_H_
#define _NODEOS_DOMAIN_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Domain an_domain_t
#define an_DomainInitFunc an_domaininit_func_t
#define an_DomainTermFunc an_domainterm_func_t
#define an_DomainStats an_domainstats_t
#define an_domainCreate an_domain_create
#define an_domainDestroy an_domain_destroy
#define an_domainId an_domain_current
#define an_domainStartThread an_domain_startthread
#define an_domainStats an_domain_stats
#endif

#include <nodeos/an_error.h>
#include <nodeos/an_impl.h>
#include <nodeos/an_cred.h>
#include <nodeos/an_resource.h>
#include <nodeos/impl/an_domain.h>

/*
 * Create a new domain with the indicated credentials and resources.
 * When the NodeOS has finished setting up the internal state of the domain,
 * a domain thread is used to asynchronously invoke the provided init function;
 * i.e., the create call may return before the init functions completes or
 * is even started.  The provided termination handler will be invoked when
 * terminating the domain, see an_domain_destroy.  Returns the new domain object
 * or zero on error.
 */
an_domain_t *an_domain_create(void *mem, an_cred_t *cred,
			  an_cpuspec_t *cpu, an_memspec_t *vm,
			  an_domaininit_func_t init, void *initarg,
			  an_domainterm_func_t term);

/*
 * Gracefully shutdown the indicated domain.
 * Invokes the registered termination callback synchronously.  Upon return
 * from that callback, internal domain destruction commences.  First, all
 * sub-domains are "internally" destroyed (i.e., no termination callbacks
 * for subdomains).  Any remaining domain threads are forcibly destroyed and
 * all domain resources are freed.  If invoked on the current domain, the
 * function will not return.
 */
an_error_t an_domain_destroy(an_domain_t *domain);

/*
 * Returns the domain identifier associated with the currently running thread.
 */
an_domain_t *an_domain_current(void);

/*
 * Activate a domain thread at a specified function.
 * The new thread starts in the indicated function with the indicated argument.
 * Returns zero if the thread launched, an error otherwise.  If out_thread is
 * non-zero, returns a pointer to the new thread there.
 */
struct an_thread;
an_error_t an_domain_startthread(an_domain_t *domain,
				 void (*f)(void *arg), void *arg,
				 struct an_thread **out_thread);

/*
 * Return statistics for a domain.
 * Return thread and other statistics for the domain in the provided buffer.
 * Returns zero on success or an error code otherwise.
 */
an_error_t an_domain_stats(an_domain_t *domain, an_domainstats_t *stats);

#endif /* _NODEOS_DOMAIN_H_ */
