/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Log API
 */
#ifndef _NODEOS_LOG_H_
#define _NODEOS_LOG_H_

#include <nodeos/an_error.h>

/*
 * Log Priorities.
 *
 * Messages of different priorities may be recorded to different logs
 * or messages below a specific priority may not be logged at all.
 * Interpretation of priorities is implementation specific.
 */
#define AN_LOG_HIGH	255	/* highest priority event */

#define	AN_LOG_EMERG	255	/* system is unusable */
#define	AN_LOG_ALERT	254	/* action must be taken immediately */
#define	AN_LOG_CRIT	253	/* critical conditions */
#define	AN_LOG_ERR	252	/* error conditions */
#define	AN_LOG_WARNING	251	/* warning conditions */
#define	AN_LOG_NOTICE	250	/* normal but significant condition */
#define	AN_LOG_INFO	249	/* informational */
#define	AN_LOG_DEBUG	248	/* debug-level messages */

#define AN_LOG_LOW	248	/* lowest priority event */

#ifdef NODEOS_SPEC_COMPAT
#define an_logWrite	an_log_write
#endif

/*
 * Log a fixed length message to the node's logfile
 * (typically the console device). 
 */
an_error_t an_log_write(int prio, void *buf, int buflen);


#endif /* _NODEOS_LOG_H_ */
