/*
 * Copyright (c) 1999, 2000, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * NodeOS Mempool API
 */
#ifndef _NODEOS_MEMPOOL_H_
#define _NODEOS_MEMPOOL_H_

#ifdef NODEOS_SPEC_COMPAT
#define an_Mempool an_mempool_t
#define an_mempoolId an_mempool_current
#define an_mempoolCreate an_mempool_create
#define an_mempoolDestroy an_mempool_destroy
#endif

#include <nodeos/an_impl.h>
#include <nodeos/an_error.h>
#include <nodeos/impl/an_mempool.h>

struct an_domain;

/*
 * Return the mempool associated with the current thread's domain.
 */
an_mempool_t *an_mempool_current(void);

/*
 * Return the mempool associated with a given domain.
 */
an_mempool_t *an_mempool_domain(struct an_domain *domain);

/*
 * Create a new mempool.
 * The mempool is to be a child of the indicated mempool (or zero if not
 * part of a hierarchy).  Func is called when the mempool exceeds its
 * allocation limit.  Returns the new mempool or zero on error.
 */
an_mempool_t *an_mempool_create(void *mem, an_mempool_t *parent,
				an_mempoolfull_func_t func);

/*
 * Destroy the indicated pool.
 * Returns an error (and doesn't destroy) if there are any domains attached
 * to the pool.
 */
an_error_t an_mempool_destroy(an_mempool_t *mpool);

/*
 * Memory allocation: simple interface.
 * Allocate the specified amount of memory charged against the given pool.
 * Returns a pointer on success, zero on failure.
 */
void *an_mempool_alloc(an_mempool_t *pool, size_t size);

/*
 * Memory deallocation: simple interface.
 * Return the indicated memory to the given pool.  ptr/size should match
 * a previously an_mempool_alloc'ed piece of memory.
 */
void an_mempool_free(an_mempool_t *pool, void *ptr, size_t size);

/*
 * Return statistics for a mempool.
 * Return memory use statistics for the given mempool in the provided buffer.
 * Returns zero on success or an error code otherwise.
 */
an_error_t an_mempool_stats(an_mempool_t *pool, an_mempoolstats_t *stats);

#endif /* _NODEOS_MEMPOOL_H_ */
