/*
 * Copyright (c) 1999-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

NODEOS_FUNC_INLINE void
ani_chan_lock(ani_chan_t *c)
{
        ani_ilock_lock(&c->lock);
}

NODEOS_FUNC_INLINE int
ani_chan_trylock(ani_chan_t *c)
{
        return ani_ilock_trylock(&c->lock);
}

NODEOS_FUNC_INLINE void
ani_chan_unlock(ani_chan_t *c)
{
        return ani_ilock_unlock(&c->lock);
}

NODEOS_FUNC_INLINE int
ani_chan_islocked(ani_chan_t *c)
{
        return ani_ilock_islocked(&c->lock);
}

NODEOS_FUNC_INLINE int
ani_chan_maxthreads(ani_chan_t *c)
{
        return c->maxthreads;
}

NODEOS_FUNC_INLINE char *
ani_chan_name(ani_chan_t *c)
{
        return (char *)(c->proto.data);
}

NODEOS_FUNC_INLINE int
ani_chan_namelen(ani_chan_t *c)
{
        return c->proto.len;
}

NODEOS_FUNC_INLINE int
ani_chan_type(ani_chan_t *c)
{
        return c->flags & CHAN_TMASK;
}

NODEOS_FUNC_INLINE void
assert_ani_chan_valid(ani_chan_t *c) 
{
	assert(c != 0);
#ifdef MEMDEBUG
        assert(c != (void *)0xdddddddd);
	assert(*(unsigned int *)c != 0xdddddddd);
#endif
}

NODEOS_FUNC_INLINE struct ani_mempool* 
ani_chan_mempool(ani_chan_t *chan) 
{
	assert(chan);
	assert(chan->domain);
	return chan->domain->mpool;
}

NODEOS_FUNC_INLINE void
ani_chan_reference(ani_chan_t* chan)
{
	assert(chan);
	chan->refs++;
}

NODEOS_FUNC_INLINE void
ani_chan_release(ani_chan_t* chan)
{
	assert(chan);
	assert(chan->refs > 0);
	if (--chan->refs == 0)
		ani_chan_cleanup(chan);
}
