/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#ifndef _NODEOS_ANI_COND_H_
#define _NODEOS_ANI_COND_H_

/*
 * ani_cond_*
 *
 * ani_cond vars wrap the ani_icond.  The wait and signal/broadcast
 * interfaces provide atomicity through the ilock in the ani_cond.
 *
 * Currently none of the ani_cond variables are inlined.
 */

#include <nodeos/an_time.h>
#include <nodeos/ani/ani_error.h>
#include <nodeos/ani/ani_ilock.h>
#include <nodeos/ani/ani_mutex.h>

typedef struct ani_cond {
	ani_icond_t	icond;
	/* 
	 * This lock protects the condition variable
	 * while the user-associated lock is released.
	 */
	ani_ilock_t	ilock;
} ani_cond_t;

/* prototypes */

ani_error_t ani_cond_init(ani_cond_t *newcond);

void ani_cond_timedwait(ani_cond_t *cond, ani_mutex_t *mutex,
			   an_timespec_t *abstime);

void ani_cond_signal(ani_cond_t *cond);

void ani_cond_broadcast(ani_cond_t *cond);

ani_error_t ani_cond_deinit(ani_cond_t *cond);

#include <nodeos/ani/ani_cond_inline.h>

#endif /* _NODEOS_ANI_COND_H_ */
