/*
 * Copyright (c) 1999-2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

NODEOS_FUNC_INLINE ani_domain_t *
ani_domain_current(void)
{
	ani_thread_t *thread = ani_thread_current();
	return thread ? ani_threadpool_domain(ani_thread_threadpool(thread)) : 0;
}

NODEOS_FUNC_INLINE struct ani_mempool *
ani_domain_mempool(ani_domain_t *domain)
{
        return (domain->mpool);
}

NODEOS_FUNC_INLINE struct ani_threadpool *
ani_domain_threadpool(ani_domain_t *domain)
{
        return (&domain->tpool);
}

NODEOS_FUNC_INLINE void
ani_domain_lock(ani_domain_t *domain)
{
        ani_ilock_lock(&domain->lock);
}

NODEOS_FUNC_INLINE void
ani_domain_unlock(ani_domain_t *domain)
{
        ani_ilock_unlock(&domain->lock);
}

NODEOS_FUNC_INLINE ani_domain_t *
ani_domain_parent(ani_domain_t *domain)
{
        return (ani_domain_t *)hnode_parent(&domain->me);
}

NODEOS_FUNC_INLINE ani_domain_t *
ani_domain_child(ani_domain_t *domain)
{
        return (ani_domain_t *)hnode_child(&domain->me);
}

NODEOS_FUNC_INLINE ani_domain_t *
ani_domain_sibling(ani_domain_t *domain)
{
        return (ani_domain_t *)hnode_sibling(&domain->me);
}

NODEOS_FUNC_INLINE void
ani_domain_reference(ani_domain_t *domain)
{
        domain->refs++;
}

NODEOS_FUNC_INLINE void
ani_domain_release(ani_domain_t *domain)
{
        if (--domain->refs == 0)
                ani_domain_cleanup(domain);
}

NODEOS_FUNC_INLINE void *
ani_domain_getapistate(ani_domain_t *domain)
{
	return domain->apistate;
}

NODEOS_FUNC_INLINE void
ani_domain_setapistate(ani_domain_t *domain, void *val)
{
	domain->apistate = val;
}

NODEOS_FUNC_INLINE void
assert_ani_domain_valid(ani_domain_t *domain) 
{
	assert(domain != 0);
#ifdef MEMDEBUG
	assert(domain != (void *)0xdddddddd);
	assert(*(unsigned int *)domain != 0xdddddddd);
#endif
}

