/*
 * Copyright (c) 1999-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_ANI_ERROR_H_
#define _NODEOS_ANI_ERROR_H_

/*
 * Moab-internal error handling types and functions.
 */

/*
 * Define this to catch subtle misuse of ani_error_t. 
 */
#undef ANI_ERROR_CHECKUSAGE

/*
 * This is currently unused:
 */
#ifdef DEBUG
#define ANI_ERROR_FULLBLOWN
#else
#undef ANI_ERROR_FULLBLOWN
#endif

#include <assert.h>

#include <nodeos/impl/decls.h>

/*
 * Currently nodeos errors are a simple integer.  The ani_error_set()
 * macro also takes a string message that can be associated with the error.
 * The message is stored in a *global* location.  Eventually, we may
 * migrate to a more robust error handling facility.  But not yet.
 */

#ifdef ANI_ERROR_CHECKUSAGE
typedef void *ani_error_t;
#else
typedef int ani_error_t;
#endif

/*
 * XXX the global error object.  Possibly should be thread-local, or
 * an explicit parameter to every function call.
 *
 * XXX these are not static so that a debugger can easily see them.
 */
extern ani_error_t ani_error_globalnum;
extern char ani_error_globalmsg[1024];

ani_error_t ani_set_error_internal(int errcode, const char *file, int line,
				   const char *msg, ...);

/*
 * Initialize an ani_error_t to a particular error.
 */
#define ani_error_set(interr, errmsg, printflikeargs...) \
	ani_set_error_internal(interr, __FILE__, __LINE__, errmsg, ## printflikeargs)

#include <nodeos/ani/ani_error_inline.h>

#endif /* _NODEOS_ANI_ERROR_H_ */
