/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_ANI_EVENT_H_
#define _NODEOS_ANI_EVENT_H_

#include <nodeos/impl/an_impl.h>
#include <nodeos/ani/ani_error.h>
#include <nodeos/ani/ani_threadwork.h>

#define ANI_EVENT_RESOLUTION

/* XXX move this into ani_time? */
typedef unsigned long ani_tick_t;

struct ani_threadpool;

struct ani_event {
	struct ani_event *next;	/* next event in the event list */
	struct ani_threadpool *tpool;/* threadpool context for event */
	ani_threadwork_t work;	/* thread work record */
	int		 flags; /* event flags */
	an_eventfunc_t	 func;	/* handler */
	void		 *farg;	/* handler argument */
	ani_tick_t	 ticks; /* time in NodeOS tick at which to fire */
	ani_tick_t	 freq;	/* frequency in ticks for periodic events */
};
typedef struct ani_event ani_event_t;

/*
 * NodeOS-private event flags.  The flags field is a combination
 * of the NodeOS-private flags (bits 0x0F00), the event type 
 * (bits 0x00F0) and the event state (bits 0x000F).
 *
 * Events have only 1 type, only 1 state, but can have any of the
 * NodeOS-private bits set.
 */
#define ANI_EV_USER     0x0000  /* A user-defined event */
#define ANI_EV_INKERN	0x0100	/* exectute in the nodeos */
#define ANI_EV_DIRECT	0x0400	/* call directly from the event loop */

/* Internal event functions */
void ani_event_setup(void);
void ani_event_shutdown(void);

void ani_event_info(unsigned long *timerres);
void ani_event_removeall(struct ani_threadpool *tpool);
void ani_event_dump(char *str);

/* API implementation functions: */
ani_error_t ani_event_init(ani_event_t *event, struct ani_threadpool *tpool);
ani_error_t ani_event_deinit(ani_event_t *event, int *oflags);
ani_error_t ani_event_schedule(ani_event_t *event, an_timespec_t *ftime,
			       enum an_eventtype type, int flags,
			       an_eventfunc_t func, void *farg);
ani_error_t ani_event_fire(ani_event_t *event,
			   an_eventfunc_t func, void *farg);
ani_error_t ani_event_deschedule(ani_event_t *event, int *oflags);
ani_error_t ani_event_redefine(ani_event_t *event,
			       an_timespec_t *otime, int *oflags,
			       an_eventfunc_t *ofunc, void **ofarg,
			       an_timespec_t *ntime, an_eventtype_t type,
			       an_eventfunc_t nfunc, void *nfarg);

#include <nodeos/ani/ani_event_inline.h>

#endif /* _NODEOS_ANI_EVENT_H_ */
