/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <assert.h>

/*
 * Ugly functions for asserting, setting, and getting the state/type out
 * of the event flags.
 *
 * We go through the trouble of combining the state and type into the
 * flags field solely so that reading the flags is atomic wrt to 
 * writing flags.  Perhaps that's just not worth it.
 */

NODEOS_FUNC_INLINE void
assert_valid_eventstate(an_eventstate_t s)
{
	assert(s == AN_EVSTATE_DESCHEDULED || s == AN_EVSTATE_SCHEDULED);
}

NODEOS_FUNC_INLINE void
assert_valid_eventtype(an_eventtype_t t)
{
	assert(t == AN_EVTYPE_UNDEFINED ||
	       t == AN_EVTYPE_PERIODIC ||
	       t == AN_EVTYPE_ONESHOT_ABSOLUTE ||
	       t == AN_EVTYPE_ONESHOT_RELATIVE);
}

NODEOS_FUNC_INLINE void
assert_valid_eventflags(int flags)
{
	an_eventstate_t s;
	an_eventtype_t t;

	assert((flags & 0xF000) == 0);
	s = (an_eventstate_t) (flags & 0x000F);
	assert_valid_eventstate(s);
	t = (an_eventtype_t) (flags & 0x00F0);
	assert_valid_eventtype(t);
}

NODEOS_FUNC_INLINE an_eventstate_t
ani_event_flagstostate(int flags)
{
	an_eventstate_t rc;

	rc = (an_eventstate_t) (flags & 0x000F);
	assert_valid_eventstate(rc);
	return rc;
}

NODEOS_FUNC_INLINE int
ani_event_flagswithstate(int flags, an_eventstate_t state)
{
	assert_valid_eventflags(flags);
	assert_valid_eventstate(state);

	flags = flags & 0xFFF0; /* clear old state */
	flags = flags | state;
	
	assert_valid_eventflags(flags);

	return flags;
}

NODEOS_FUNC_INLINE an_eventtype_t
ani_event_flagstotype(int flags)
{
	an_eventtype_t rc;

	assert_valid_eventflags(flags);
	rc = (an_eventtype_t) (flags & 0x00F0);
	assert_valid_eventtype(rc);
	return rc;
}

NODEOS_FUNC_INLINE int
ani_event_flagswithtype(int flags, an_eventtype_t type)
{
	assert_valid_eventflags(flags);
	assert_valid_eventtype(type);

	flags = flags & 0xFF0F; /* clear old type */
	flags = flags | type;
	
	assert_valid_eventflags(flags);
	return flags;
}

NODEOS_FUNC_INLINE an_eventtype_t
ani_event_type(ani_event_t *event)
{
	assert(event);
	return ani_event_flagstotype(event->flags);
}

NODEOS_FUNC_INLINE void
ani_event_settype(ani_event_t *event, an_eventtype_t type)
{
	assert(event);
	event->flags = ani_event_flagswithtype(event->flags, type);
}

NODEOS_FUNC_INLINE int
ani_event_flags(ani_event_t *event)
{
	assert(event);
	return event->flags;
}

NODEOS_FUNC_INLINE an_eventstate_t
ani_event_state(ani_event_t *event)
{
	assert(event);
	return ani_event_flagstostate(event->flags);
}

NODEOS_FUNC_INLINE void
ani_event_setstate(ani_event_t *event, an_eventstate_t state)
{
	assert(event);
	event->flags = ani_event_flagswithstate(event->flags, state);
}
