/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/* inline functions */

NODEOS_FUNC_INLINE ani_error_t 
ani_file_close(ani_file_t *file) 
{
	ani_error_t err;

	if (close(*file) == -1)
		err = ani_error_set(errno, "native close call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_fsync(ani_file_t *file)
{
	ani_error_t err;

        if (fsync(*file) == -1)
		err = ani_error_set(errno, "native fsync call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_truncate(ani_file_t *file, an_offset_t offset)
{
	ani_error_t err;

        if (ftruncate(*file, offset) == -1)
		err = ani_error_set(errno, "native ftruncate call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_lseek(ani_file_t *file, an_offset_t length, an_whence_t whence,
	       an_offset_t *retoff)
{
	ani_error_t err;

	if ((*retoff = lseek(*file, length, whence)) == -1)
		err = ani_error_set(errno, "native lseek call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_mkdir(const char *path, an_acd_t *acd) 
{
	ani_error_t err;

	if (mkdir(path, acd->st_mode) == -1)
		err = ani_error_set(errno, "native mkdir call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_read(ani_file_t *file, an_size_t nbytes, void *buf, an_size_t *retlen)
{
	ani_error_t err;

	if ((*retlen = read(*file, buf, nbytes)) == -1)
		err = ani_error_set(errno, "native read call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_rename(const char *from, const char *to)
{
	ani_error_t err;

        if (rename(from, to) == -1)
		err = ani_error_set(errno, "native rename call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_write(ani_file_t *file, an_size_t length, void *buf, an_size_t *retval)
{
	ani_error_t err;

	if ((*retval = write(*file, buf, length)) == -1)
		err = ani_error_set(errno, "native write call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_unlink(const char *path)
{
	ani_error_t err;

	if (unlink(path) == -1)
		err = ani_error_set(errno, "native unlink call failed");
	else
		err = ani_error_clear();

	return err;
}

NODEOS_FUNC_INLINE ani_error_t 
ani_file_open(const char *path, int flags, an_acd_t *acd, ani_file_t *retval)
{
	ani_error_t err;

	if ((*retval = open(path, flags, acd->st_mode)) == -1)
		err = ani_error_set(errno, "native open call failed");
	else
		err = ani_error_clear();

	return err;
}

