/*
 * Copyright (c) 1999-2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _MEMPOOL_H_
#define _MEMPOOL_H_

#include <nodeos/impl/an_impl.h>
#include <nodeos/ani/ani_error.h>
#include <nodeos/ani/ani_hnode.h>

#define ANI_MEMPOOL_CHUNKSIZE	4096

/*
 * The mempool: aggregates the memory resources from multiple domains.
 */

struct ani_mpoolstats {
	unsigned int exchunks;	/* bytes of mem alloced through API */
	unsigned int imchunks;	/* bytes of mem alloced via infrastructure */
	int numdomains;		/* number of domains attached */
};
typedef struct ani_mpoolstats ani_mpoolstats_t;

struct ani_mempool {
	struct hnode me;	/* must be first */
	struct hnode domainlist;/* all domains associated with this pool */
	int physcurrent;	/* chunks in use by domains in this pool */
	int physlimit;		/* chunks available to domains in this pool */
	an_mempoolfull_func_t callback; /* reclaimation callback */
	ani_mpoolstats_t stats;	/* statistics */
};
typedef struct ani_mempool ani_mempool_t;

struct ani_domain;

void ani_mempool_setup(ani_mempool_t *root_mempool);
void ani_mempool_shutdown(void);

/*
 * Get the mempool associated with the current thread's domain.
 */
ani_mempool_t *ani_mempool_current(void);

/*
 * Return the root mempool
 */
ani_mempool_t *ani_mempool_rootpool(void);

/*
 * Initialize a new mempool object with no associated domains.  The parent
 * parameter defines this pool's position in the pool hierarchy.
 * The callback routine is invoked whenever the NodeOS detects that
 * the indicated pool is over its memory limit.  Parameters to the
 * callback function include the pool and some indication of how
 * much memory needs to be returned.  The callback function should
 * use unmapregion to free up memory.
 */
ani_error_t ani_mempool_init(ani_mempool_t *mpool, ani_mempool_t *parent,
			     an_mempoolfull_func_t callback);

/*
 * Destroy a mempool.  All domains that were in the pool are implicitly
 * removed (see an_mem_removedomain) and all child pools are (recursively)
 * destroyed.
 */
ani_error_t ani_mempool_deinit(ani_mempool_t *pool);

/*
 * Donates the memory resources associated with a domain to the pool.
 */
void ani_mempool_adddomain(ani_mempool_t *pool, struct ani_domain *domain);

/*
 * Removes the memory resources associated with a domain from the pool.
 */
void ani_mempool_removedomain(ani_mempool_t *pool, struct ani_domain *domain);

/*
 * Simple malloc/free allocation interface
 */
void *ani_mempool_alloc(ani_mempool_t *pool, size_t size);
void ani_mempool_free(ani_mempool_t *pool, void *ptr, size_t size);

/*
 * Return stats
 */
ani_error_t ani_mempool_stats(ani_mempool_t *pool, an_mempoolstats_t *stats);

#endif /* _MEMPOOL_H_ */
