/*
 * Copyright (c) 1999-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/* inline functions */

NODEOS_FUNC_INLINE ani_thread_t *
ani_thread_current(void)
{
	ani_thread_t *thread;

	/*
	 * In the current implementation has no control over scheduling
	 * of threads, so we have to query the pthread state.
	 */
	thread = (ani_thread_t *)pthread_getspecific(ani_nodeos_thread);
	assert(thread == 0 || thread->id == pthread_self());
	return thread;
}

NODEOS_FUNC_INLINE struct ani_threadpool *
ani_thread_threadpool(ani_thread_t *thread)
{
	assert(thread != NULL);
	return thread->pool;
}

NODEOS_FUNC_INLINE void
ani_thread_lock(ani_thread_t *thread)
{
        ani_ilock_lock(&thread->lock);
}

NODEOS_FUNC_INLINE void
ani_thread_unlock(ani_thread_t *thread)
{
        ani_ilock_unlock(&thread->lock);
}

NODEOS_FUNC_INLINE void
assert_ani_thread_valid(ani_thread_t *thread)
{
	assert(thread != NULL);
#ifdef MEMDEBUG
	assert(thread != (void *)0xdddddddd);
	assert(*(unsigned int *)thread != 0xdddddddd);
#endif
	assert(thread->pool != NULL);
}

