/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

struct ani_domain;

/* cond inline functions */

/* XXX NODEOS_CREATE_COND is not inlined due to cyclic include problems */
/* XXX NODEOS_DESTROY_COND is not inlined due to cyclic include problems */

NODEOS_FUNC_INLINE struct ani_cond *
NODEOS_INTERN_COND(struct ani_domain *domain, an_cond_t *xcond)
{
	assert(xcond != 0);
	return &xcond->cond;
}

NODEOS_FUNC_INLINE an_cond_t *
NODEOS_EXTERN_COND(struct ani_domain *domain, struct ani_cond *icond)
{
	assert(icond != 0);
	return (an_cond_t *)icond;
}

NODEOS_FUNC_INLINE void 
an_cond_wait(an_cond_t *c, an_mutex_t *m)
{
	NODEOS_ENTER;
	ani_cond_timedwait(&c->cond, NODEOS_INTERN_MUTEX(0, m), 0);
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_timedwait(an_cond_t *c, an_mutex_t *m, an_timespec_t *abstime)
{
	NODEOS_ENTER;
	ani_cond_timedwait(&c->cond, NODEOS_INTERN_MUTEX(0, m), abstime);
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_signal(an_cond_t *c)
{
	NODEOS_ENTER;
	ani_cond_signal(&c->cond);
	NODEOS_LEAVE;
}

NODEOS_FUNC_INLINE void
an_cond_broadcast(an_cond_t *c)
{
	NODEOS_ENTER;
	ani_cond_broadcast(&c->cond);
	NODEOS_LEAVE;
}
