/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

struct ani_domain;

/* XXX NODEOS_CREATE_EVENT is not inlined due to cyclic include problems */
/* XXX NODEOS_DESTROY_EVENT is not inlined due to cyclic include problems */

NODEOS_FUNC_INLINE ani_event_t *
NODEOS_INTERN_EVENT(struct ani_domain *domain, an_event_t *event)
{
	assert(event != 0);
	return &event->event;
}

NODEOS_FUNC_INLINE an_event_t *
NODEOS_EXTERN_EVENT(struct ani_domain *domain, ani_event_t *ievent)
{
	assert(ievent != 0);
	return (an_event_t *)ievent;
}

/* an_event_create() is not inlined. */
/* an_event_destroy() is not inlined. */

NODEOS_FUNC_INLINE an_error_t 
an_event_schedule(an_event_t *event, an_timespec_t *etime,
		  an_eventtype_t type, an_eventfunc_t func, void *farg)
{
	ani_error_t err;

	NODEOS_ENTER;
	err = ani_event_schedule(&event->event, etime, type, ANI_EV_USER,
				 func, farg);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}


NODEOS_FUNC_INLINE an_error_t 
an_event_fire(an_event_t *event, an_eventfunc_t func, void *arg)
{
	ani_error_t err;

	NODEOS_ENTER;
	err = ani_event_fire(&event->event, func, arg);
	NODEOS_LEAVE;

	return NODEOS_EXTERN_ERROR(err);
}


NODEOS_FUNC_INLINE an_error_t 
an_event_status(an_event_t *event, an_eventtype_t *type,
		an_eventstate_t *state)
{
	int iflags;

	LW_NODEOS_ENTER;
	iflags = ani_event_flags(&event->event);
	LW_NODEOS_LEAVE;
	
	if (type)
		*type = ani_event_flagstotype(iflags);
	if (state)
		*state = ani_event_flagstostate(iflags);

	return 0;
}

NODEOS_FUNC_INLINE an_error_t 
an_event_deschedule(an_event_t *event, an_eventtype_t *type,
		    an_eventstate_t *state)
{
	ani_error_t err;
	int iflags;

	NODEOS_ENTER;
	err = ani_event_deschedule(&event->event, &iflags);
	NODEOS_LEAVE;

	if (!ani_error_occured(err)) {
		if (type)
			*type = ani_event_flagstotype(iflags);
		if (state)
			*state = ani_event_flagstostate(iflags);
	}
	
	return NODEOS_EXTERN_ERROR(err);
}

/* an_event_redefine() is not inlined. */

NODEOS_FUNC_INLINE unsigned long
an_event_granularity(void)
{
	unsigned long res;

	LW_NODEOS_ENTER;
	ani_event_info(&res);
	LW_NODEOS_LEAVE;

	return res;
}
