/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

struct ani_domain;

/* an_file inline functions */

/* XXX NODEOS_CREATE_FILE is not inlined due to cyclic include problems */
/* XXX NODEOS_DESTROY_FILE is not inlined due to cyclic include problems */

NODEOS_FUNC_INLINE ani_file_t *
NODEOS_INTERN_FILE(struct ani_domain *domain, an_file_t *xfile)
{
	assert(xfile != 0);
	return &xfile->file;
}

NODEOS_FUNC_INLINE an_file_t *
NODEOS_EXTERN_FILE(struct ani_domain *domain, ani_file_t *ifile)
{
	assert(ifile != 0);
	return (an_file_t *)ifile;
}

NODEOS_FUNC_INLINE an_error_t 
an_file_close(an_file_t *file) 
{
	an_error_t err;

	NODEOS_ENTER;
	err = ani_file_close(&file->file);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_fsync(an_file_t *file)
{
	an_error_t err;

	NODEOS_ENTER;
	err = ani_file_fsync(&file->file);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_truncate(an_file_t *file, an_offset_t offset)
{
	an_error_t err;

	NODEOS_ENTER;
	err = ani_file_truncate(&file->file, offset);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_lseek(an_file_t *file, an_offset_t length, an_whence_t whence,
	      an_offset_t *retoff)
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyout of info needed in our implementation */
	err = ani_file_lseek(&file->file, length, whence, retoff);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_mkdir(const char *path, an_acd_t *acd) 
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyin of info needed in our implementation */
	err = ani_file_mkdir(path, acd);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_read(an_file_t *file, an_size_t nbytes, void *buf, an_size_t *retlen)
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyout of info needed in our implementation */
	err = ani_file_read(&file->file, nbytes, buf, retlen);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_rename(const char *from, const char *to)
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyin of info needed in our implementation */
	err = ani_file_rename(from, to);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_write(an_file_t *file, an_size_t length, void *buf, an_size_t *retval)
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyin of info needed in our implementation */
	err = ani_file_write(&file->file, length, buf, retval);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_unlink(const char *path)
{
	an_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyin of info needed in our implementation */
	err = ani_file_unlink(path);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_stat(const char *path, an_stat_t *retval)
{
	ani_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyout of info needed in our implementation */
	err = ani_file_stat(path, retval);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

NODEOS_FUNC_INLINE an_error_t 
an_file_fstat(an_file_t *file, an_stat_t *retval)
{
	ani_error_t err;

	NODEOS_ENTER;
	/* N.B. no copyout of info needed in our implementation */
	err = ani_file_fstat(&file->file, retval);
	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}
