/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/* an_pbuf inline functions */

struct ani_domain;

/* XXX NODEOS_CREATE_PBUF is not inlined due to cyclic include problems */
/* XXX NODEOS_DESTROY_PBUF is not inlined due to cyclic include problems */

NODEOS_FUNC_INLINE struct ani_pbuf *
NODEOS_INTERN_PBUF(struct ani_domain *domain, an_pbuf_t *xpbuf)
{
	assert(xpbuf != 0);
	return &xpbuf->pbuf;
}

NODEOS_FUNC_INLINE an_pbuf_t *
NODEOS_EXTERN_PBUF(struct ani_domain *domain, struct ani_pbuf *ipbuf)
{
	assert(ipbuf != 0);
	return (an_pbuf_t *)ipbuf;
}

NODEOS_FUNC_INLINE void *
an_pbuf_data(an_pbuf_t *pbuf)
{
	void *data;

	LW_NODEOS_ENTER;
	data = ani_pbuf_data(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return data;
}

NODEOS_FUNC_INLINE int
an_pbuf_size(an_pbuf_t *pbuf)
{
	int size;

	LW_NODEOS_ENTER;
	size = (int)ani_pbuf_size(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return size;
}

NODEOS_FUNC_INLINE int
an_pbuf_offset(an_pbuf_t *pbuf)
{
	int offset;

	LW_NODEOS_ENTER;
	offset = (int)ani_pbuf_offset(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return offset;
}

NODEOS_FUNC_INLINE int
an_pbuf_bufsize(an_pbuf_t *pbuf)
{
	int size;

	LW_NODEOS_ENTER;
	size = (int)ani_pbuf_bufsize(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return size;
}

/*
 * Determine if a pbuf is currently busy.
 *
 * XXX the busy state of a pbuffer can be cleared at interrupt time so there
 * is an obvious race here.  This shouldn't be a problem, but if it is,
 * ani_pbuf_isbusy should be responsible for ensuring the desired semantics.
 */
NODEOS_FUNC_INLINE int
an_pbuf_isbusy(an_pbuf_t *pbuf)
{
	int isbusy;

	LW_NODEOS_ENTER;
	isbusy = ani_pbuf_isbusy(&pbuf->pbuf);
	LW_NODEOS_LEAVE;

	return isbusy;
}

