/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Implementation specific statistics-gathering structs.
 */
#ifndef _NODEOS_IMPL_STATS_H_
#define _NODEOS_IMPL_STATS_H_

typedef unsigned long long an_statval_t;

/*
 * Domain statistics
 */
typedef struct {
	an_statval_t curthreads;	/* current threads running */
	an_statval_t maxthreads;	/* max threads running */
	an_statval_t activations;	/* total thread activations */
	an_statval_t cpuunits;		/* units for cpuused field */
	an_statval_t cpuused;		/* total microseconds of CPU used */
	an_statval_t memused;		/* total bytes used */
} an_domainstats_t;

/* domain cpu usage units */
#define NODEOS_CPU_USECS	1	/* microseconds */
#define NODEOS_CPU_CYCLES	2	/* machine cycles */

/*
 * Channel statistics
 */
typedef struct {
	an_statval_t inpackets;		/* calls to recv function */
	an_statval_t inbytes;		/* and number of bytes received */
	an_statval_t outpackets;	/* calls to ani_chan_send */
	an_statval_t outbytes;		/* and number of bytes sent */
	an_statval_t droppackets;	/* total packets dropped by NodeOS.. */
	an_statval_t dropnobufs;	/* ..due to no memory resources */
	an_statval_t dropfailedbuf;	/* ..due to failed buffer setup */
	an_statval_t dropbadpacket;	/* ..due to protocol processing */
	an_statval_t droplocked;	/* ..due to locked resource */
	an_statval_t dropbytes;		/* bytes dropped */
	an_statval_t inbufqueued;	/* in packets queued for a thread */
	an_statval_t inbufcopies;	/* in packets needing extra copy */
	an_statval_t maxthreads;	/* max threads active */
	an_statval_t outbufcopies;	/* out packets needing extra copy */
} an_chanstats_t;

/*
 * Mempool statistics
 */
typedef struct {
	an_statval_t exchunks;		/* chunks of explicitly alloced mem */
	an_statval_t imchunks;		/* chunks of implicitly alloced mem */
	an_statval_t chunksize;		/* size of reported chunks */
	an_statval_t numdomains;		/* number of domains in mempool */
} an_mempoolstats_t;

#endif /* _NODEOS_IMPL_STATS_H_ */
