/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef __CLICK_HH__
#define __CLICK_HH__

/* 
 * The graph structure. All that we need is space for 3 pointers. 
 */
typedef struct anlib_clickgraphdummy {
	int *d1;
	int *d2;
	int *d3;
} anlib_clickgraphdummy_t;

typedef struct anlib_clickgraph anlib_clickgraph_t;         
        
/* Initialize the graph, check the configuration for validity and
 *  create a router object. 
 */
anlib_clickgraph_t* 
anlib_clickgraph_create (void *mem, char *config);     
        
/* A click graph has at least 2 end-points (input and output) and these can
 *  connected either to
 *  a. A click channel (and in or out channel)
 *  b. ee defined function (This would facilitate inserting click
 *  graph anywhere in the processing path)        
 */
/*	The function below would return a function and its argument, which can be 
 *	directly passed to an_inchan_create. The returned function is essentially
 *	an entry point into the click graph.
 *	If the input is not connected to a channel, the ee can call this 
 *  function to pass pbuf into the click graph. 
 */
an_error_t
anlib_clickgraph_lookupInput (anlib_clickgraph_t *graph, 
								const char *name,
								an_chanrecvfunc_t *recvfunc, 
								void **funcarg);

/* Signature of a input function that would accept a packet */
typedef struct p Packet;
typedef void (*anlib_recvPCGfunc_t)(Packet *p, void *obj, int foo);

an_error_t
anlib_clickgraph_lookupInputPCG (anlib_clickgraph_t *graph, 
								const char *name,
								anlib_recvPCGfunc_t *recvfunc, 
								void **funcarg);

/* Signature of a send function. This is basically the signature of
   an_chan_send function */
typedef an_error_t (*an_sendfunc_t)(an_chan_t *chan, an_pbuf_t *pbuf);

/* 
 * Use anlib_clickgraph_attachOutput to specify an exit function from the click
 * graph. After finishing its processing the click graph would call this method
 * to get the packet out of the click graph.
 */
an_error_t 
anlib_clickgraph_attachOutput(anlib_clickgraph_t* graph,
                             	const char* name,
                              	an_sendfunc_t outputFunc,
                           		an_chan_t *OutFuncArg);


an_error_t 
anlib_clickgraph_attachOutputPCG(anlib_clickgraph_t* graph,
                             	const char* name,
                              	anlib_recvPCGfunc_t outputFunc,
                           		an_chan_t *OutFuncArg);

an_error_t 
anlib_clickgraph_attachOutputA(anlib_clickgraph_t* graph,
                             	const char* name,
                              	an_chanrecvfunc_t outputFunc,
                           		an_chan_t *OutFuncArg);


/* We don't need this graph anymore. Do the cleanup */
an_error_t
anlib_clickgraph_destroy (anlib_clickgraph_t *graph);

#endif //__CLICK_HH__
