/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * an_cred inline functions
 *
 * XXX these functions are here rather than in the nodeos/impl directory
 * as they should not be exposed in the API.
 */

/*
 * Create a new object using the resources of the indicated domain.
 * Mem is the external memory which should be initialized to reference
 * the new object.  If mem is zero, the external object will be allocated.
 * Returns a pointer to the internal object, or zero on failure.
 */
NODEOS_FUNC_INLINE ani_cred_t *
NODEOS_CREATE_CRED(ani_domain_t *hdomain, an_cred_t *xcred)
{
	ani_error_t err;
	ani_cred_t *icred;

	err = nodeos_obj_alloc(hdomain, xcred, NODEOS_TYPE_CRED,
			       sizeof(an_cred_t), sizeof(ani_cred_t),
			       (void **)&icred);

	return ani_error_occured(err) ? 0 : icred;
}

/*
 * Destroy an object
 */
NODEOS_FUNC_INLINE void
NODEOS_DESTROY_CRED(ani_domain_t *hdomain, ani_cred_t *icred)
{
	nodeos_obj_free(hdomain, icred, NODEOS_TYPE_CRED,
			sizeof(an_cred_t), sizeof(ani_cred_t));
}

/*
 * Convert an external representation into an internal one.
 */
NODEOS_FUNC_INLINE ani_cred_t *
NODEOS_INTERN_CRED(ani_domain_t *hdomain, an_cred_t *xcred)
{
	ani_cred_t *icred;
	ani_error_t err;

	assert(xcred != 0);

	err = nodeos_obj_intern(hdomain, xcred, xcred->index,
				NODEOS_TYPE_CRED, (void **)&icred);

	return ani_error_occured(err) ? 0 : icred;
}

/*
 * Convert an internal representation into an external one.
 */
NODEOS_FUNC_INLINE an_cred_t *
NODEOS_EXTERN_CRED(ani_domain_t *hdomain, ani_cred_t *icred)
{
	an_cred_t *xcred;
	ani_error_t err;
	int ix;

	assert(icred != 0);

	err = nodeos_obj_extern(hdomain, icred,
				NODEOS_TYPE_CRED, (void **)&xcred, &ix);
	if (ani_error_occured(err))
		return 0;

	xcred->index = ix;
	return xcred;
}

