/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * an_demuxkey inline functions
 *
 * XXX these functions are here rather than in the nodeos/impl directory
 * as they should not be exposed in the API.
 */

/*
 * Create a new object using the resources of the indicated domain.
 * Mem is the external memory which should be initialized to reference
 * the new object.  If mem is zero, the external object will be allocated.
 * Returns a pointer to the internal object, or zero on failure.
 */
NODEOS_FUNC_INLINE ani_demuxkey_t *
NODEOS_CREATE_DEMUXKEY(ani_domain_t *hdomain, an_demuxkey_t *xkey)
{
	ani_error_t err;
	ani_demuxkey_t *ikey;

	err = nodeos_obj_alloc(hdomain, xkey, NODEOS_TYPE_DEMUXKEY,
			       sizeof(an_demuxkey_t), sizeof(ani_demuxkey_t),
			       (void **)&ikey);

	return ani_error_occured(err) ? 0 : ikey;
}

/*
 * Destroy an object
 */
NODEOS_FUNC_INLINE void
NODEOS_DESTROY_DEMUXKEY(ani_domain_t *hdomain, ani_demuxkey_t *ikey)
{
	nodeos_obj_free(hdomain, ikey, NODEOS_TYPE_DEMUXKEY,
			sizeof(an_demuxkey_t), sizeof(ani_demuxkey_t));
}

/*
 * Convert an external representation into an internal one.
 */
NODEOS_FUNC_INLINE ani_demuxkey_t *
NODEOS_INTERN_DEMUXKEY(ani_domain_t *hdomain, an_demuxkey_t *xkey)
{
	ani_demuxkey_t *ikey;
	ani_error_t err;

	assert(xkey != 0);

	err = nodeos_obj_intern(hdomain, xkey, xkey->index,
				NODEOS_TYPE_DEMUXKEY, (void **)&ikey);

	return ani_error_occured(err) ? 0 : ikey;
}

/*
 * Convert an internal representation into an external one.
 */
NODEOS_FUNC_INLINE an_demuxkey_t *
NODEOS_EXTERN_DEMUXKEY(ani_domain_t *hdomain, ani_demuxkey_t *ikey)
{
	an_demuxkey_t *xkey;
	ani_error_t err;
	int ix;

	assert(ikey != 0);

	err = nodeos_obj_extern(hdomain, ikey,
				NODEOS_TYPE_DEMUXKEY, (void **)&xkey, &ix);
	if (ani_error_occured(err))
		return 0;

	xkey->index = ix;
	return xkey;
}

