/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * an_event inline functions
 *
 * XXX these functions are here rather than in the nodeos/impl directory
 * as they should not be exposed in the API.
 */

/*
 * Create a new object using the resources of the indicated domain.
 * Mem is the external memory which should be initialized to reference
 * the new object.  If mem is zero, the external object will be allocated.
 * Returns a pointer to the internal object, or zero on failure.
 */
NODEOS_FUNC_INLINE ani_event_t *
NODEOS_CREATE_EVENT(ani_domain_t *hdomain, an_event_t *xevent)
{
	ani_error_t err;
	ani_event_t *ievent;

	err = nodeos_obj_alloc(hdomain, xevent, NODEOS_TYPE_EVENT,
			       sizeof(an_event_t), sizeof(ani_event_t),
			       (void **)&ievent);

	return ani_error_occured(err) ? 0 : ievent;
}

/*
 * Destroy an object
 */
NODEOS_FUNC_INLINE void
NODEOS_DESTROY_EVENT(ani_domain_t *hdomain, ani_event_t *ievent)
{
	nodeos_obj_free(hdomain, ievent, NODEOS_TYPE_EVENT,
			sizeof(an_event_t), sizeof(ani_event_t));
}

/*
 * Convert an external representation into an internal one.
 */
NODEOS_FUNC_INLINE ani_event_t *
NODEOS_INTERN_EVENT(ani_domain_t *hdomain, an_event_t *xevent)
{
	ani_event_t *ievent;
	ani_error_t err;

	assert(xevent != 0);

	err = nodeos_obj_intern(hdomain, xevent, xevent->index,
				NODEOS_TYPE_EVENT, (void **)&ievent);

	return ani_error_occured(err) ? 0 : ievent;
}

/*
 * Convert an internal representation into an external one.
 */
NODEOS_FUNC_INLINE an_event_t *
NODEOS_EXTERN_EVENT(ani_domain_t *hdomain, ani_event_t *ievent)
{
	an_event_t *xevent;
	ani_error_t err;
	int ix;

	assert(ievent != 0);

	err = nodeos_obj_extern(hdomain, ievent,
				NODEOS_TYPE_EVENT, (void **)&xevent, &ix);
	if (ani_error_occured(err))
		return 0;

	xevent->index = ix;
	return xevent;
}

