/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * an_mempool inline functions
 *
 * XXX these functions are here rather than in the nodeos/impl directory
 * as they should not be exposed in the API.
 */

/*
 * Create a new object using the resources of the indicated domain.
 * Mem is the external memory which should be initialized to reference
 * the new object.  If mem is zero, the external object will be allocated.
 * Returns a pointer to the internal object, or zero on failure.
 */
NODEOS_FUNC_INLINE ani_mempool_t *
NODEOS_CREATE_MEMPOOL(ani_domain_t *hdomain, an_mempool_t *xmpool)
{
	ani_error_t err;
	ani_mempool_t *impool;

	err = nodeos_obj_alloc(hdomain, xmpool, NODEOS_TYPE_MEMPOOL,
			       sizeof(an_mempool_t), sizeof(ani_mempool_t),
			       (void **)&impool);

	return ani_error_occured(err) ? 0 : impool;
}

/*
 * Destroy an object
 */
NODEOS_FUNC_INLINE void
NODEOS_DESTROY_MEMPOOL(ani_domain_t *hdomain, ani_mempool_t *impool)
{
	nodeos_obj_free(hdomain, impool, NODEOS_TYPE_MEMPOOL,
			sizeof(an_mempool_t), sizeof(ani_mempool_t));
}

/*
 * Convert an external representation into an internal one.
 */
NODEOS_FUNC_INLINE ani_mempool_t *
NODEOS_INTERN_MEMPOOL(ani_domain_t *hdomain, an_mempool_t *xmpool)
{
	ani_mempool_t *impool;
	ani_error_t err;

	assert(xmpool != 0);

	err = nodeos_obj_intern(hdomain, xmpool, xmpool->index,
				NODEOS_TYPE_MEMPOOL, (void **)&impool);

	return ani_error_occured(err) ? 0 : impool;
}

/*
 * Convert an internal representation into an external one.
 */
NODEOS_FUNC_INLINE an_mempool_t *
NODEOS_EXTERN_MEMPOOL(ani_domain_t *hdomain, ani_mempool_t *impool)
{
	an_mempool_t *xmpool;
	ani_error_t err;
	int ix;

	assert(impool != 0);

	err = nodeos_obj_extern(hdomain, impool,
				NODEOS_TYPE_MEMPOOL, (void **)&xmpool, &ix);
	if (ani_error_occured(err))
		return 0;

	xmpool->index = ix;
	return xmpool;
}

