/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * an_pbuf inline functions
 *
 * XXX these functions are here rather than in the nodeos/impl directory
 * as they should not be exposed in the API.
 */

/*
 * Create a new object using the resources of the indicated domain.
 * Mem is the external memory which should be initialized to reference
 * the new object.  If mem is zero, the external object will be allocated.
 * Returns a pointer to the internal object, or zero on failure.
 */
NODEOS_FUNC_INLINE ani_pbuf_t *
NODEOS_CREATE_PBUF(ani_domain_t *hdomain, an_pbuf_t *xpbuf)
{
	ani_error_t err;
	ani_pbuf_t *ipbuf;

	err = nodeos_obj_alloc(hdomain, xpbuf, NODEOS_TYPE_PBUF,
			       sizeof(an_pbuf_t), sizeof(ani_pbuf_t),
			       (void **)&ipbuf);

	return ani_error_occured(err) ? 0 : ipbuf;
}

/*
 * Destroy an object
 */
NODEOS_FUNC_INLINE void
NODEOS_DESTROY_PBUF(ani_domain_t *hdomain, ani_pbuf_t *ipbuf)
{
	nodeos_obj_free(hdomain, ipbuf, NODEOS_TYPE_PBUF,
			sizeof(an_pbuf_t), sizeof(ani_pbuf_t));
}

/*
 * Convert an external representation into an internal one.
 */
NODEOS_FUNC_INLINE ani_pbuf_t *
NODEOS_INTERN_PBUF(ani_domain_t *hdomain, an_pbuf_t *xpbuf)
{
	ani_pbuf_t *ipbuf;
	ani_error_t err;

	assert(xpbuf != 0);

	err = nodeos_obj_intern(hdomain, xpbuf, xpbuf->index,
				NODEOS_TYPE_PBUF, (void **)&ipbuf);

	return ani_error_occured(err) ? 0 : ipbuf;
}

/*
 * Convert an internal representation into an external one.
 */
NODEOS_FUNC_INLINE an_pbuf_t *
NODEOS_EXTERN_PBUF(ani_domain_t *hdomain, ani_pbuf_t *ipbuf)
{
	an_pbuf_t *xpbuf;
	ani_error_t err;
	int ix;

	assert(ipbuf != 0);

	err = nodeos_obj_extern(hdomain, ipbuf,
				NODEOS_TYPE_PBUF, (void **)&xpbuf, &ix);
	if (ani_error_occured(err))
		return 0;

	xpbuf->index = ix;
	return xpbuf;
}

