/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifndef _NODEOS_SRC_NODEOS_H_
#define _NODEOS_SRC_NODEOS_H_

/*
 * NodeOS object types
 */
#define NODEOS_TYPE_NONE	0
#define NODEOS_TYPE_DOMAIN	1
#define NODEOS_TYPE_THREAD	2
#define NODEOS_TYPE_CRED	3
#define NODEOS_TYPE_MEMPOOL	4
#define NODEOS_TYPE_CHAN	5
#define NODEOS_TYPE_DEMUXKEY	6
#define NODEOS_TYPE_PBUF	7
#define NODEOS_TYPE_MUTEX	8
#define NODEOS_TYPE_COND	9
#define NODEOS_TYPE_EVENT	10
#define NODEOS_TYPE_FILE	11

#define NODEOS_FLAG_ALLOCEXT	1	/* NodeOS allocated external object */
#define NODEOS_FLAG_ALLOCINT	2	/* NodeOS allocated internal object */

#define NODEOS_DESC_GROWSIZE	128
#define NODEOS_DESC_MAXSIZE	(128*1024)

struct nodeos_dent {
	short	type;	/* object type */
	short	flags;	/* flags */
	void	*iptr;	/* pointer to internal object */
	void	*xptr;	/* pointer to external object */
};
typedef struct nodeos_dent nodeos_dent_t;

struct ani_domain;
struct ani_mempool;

struct nodeos_dtab {
	int	len;			/* current length of table in entries */
	int	maxlen;			/* max length of table in entries */
	struct ani_mempool *pool;	/* mempool to grow table from */
	nodeos_dent_t *ents;		/* table */
};
typedef struct nodeos_dtab nodeos_dtab_t;

/*
 * NodeOS API conversion functions
 */
nodeos_dtab_t *nodeos_desc_allocroottab(struct ani_mempool *mpool,
					int startlen, int maxlen);
nodeos_dtab_t *nodeos_desc_alloctab(struct ani_domain *domain,
				    int startlen, int maxlen);
void nodeos_desc_freetab(nodeos_dtab_t *dtab);
void nodeos_desc_inserttab(nodeos_dtab_t *dtab, void *xobj, void *iobj,
			   int ntype, int *indexp);
ani_error_t nodeos_obj_alloc(struct ani_domain *domain, void *mem, int ntype,
			     int xsize, int isize, void **iobjp);
void nodeos_obj_free(struct ani_domain *domain, void *iobj, int ntype,
		     int xsize, int isize);

ani_error_t nodeos_obj_intern(struct ani_domain *domain, void *xobj, int ix,
			      int ntype, void **iobjp);
ani_error_t nodeos_obj_extern(struct ani_domain *domain, void *iobj, int ntype,
			      void **xobjp, int *ixp);

#endif



