/*
 * Copyright (c) 2000-2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * File system interface
 */
#include <nodeos/an_fs.h>
#include <nodeos/an_error.h>

#include <nodeos/ani/ani_nodeos.h>
#include <nodeos/ani/ani_domain.h>
#include <nodeos/ani/ani_mempool.h>
#include <nodeos/ani/ani_fs.h>

ani_file_t *
NODEOS_CREATE_FILE(ani_domain_t *domain, an_file_t *xfile)
{
	if (xfile == 0) {
		xfile = ani_mempool_alloc(ani_domain_mempool(domain),
					  sizeof(an_file_t));
		if (xfile == 0)
			return 0;
		xfile->nodeosobj = xfile;
	} else
		xfile->nodeosobj = 0;

	return &xfile->file;
}

void
NODEOS_DESTROY_FILE(ani_domain_t *domain, ani_file_t *ifile)
{
	an_file_t *xfile = (an_file_t *)ifile;

	assert(xfile != 0);
	if (xfile->nodeosobj != 0) {
		assert(xfile->nodeosobj == xfile);
		ani_mempool_free(ani_domain_mempool(domain), xfile,
				 sizeof(an_file_t));
	}
}

an_error_t 
an_file_open(const char *path, int flags, an_acd_t *acd, an_file_t **retval)
{
	ani_error_t err;
	ani_file_t *ifile;
	ani_domain_t *hdomain;
	void *mem;

	NODEOS_ENTER;

	hdomain = ani_domain_current();

	/*
	 * Create a new object
	 */
	mem = *retval;
	ifile = NODEOS_CREATE_FILE(hdomain, mem);
	if (ifile == 0) {
		err = ani_error_set(AN_ERR_INVALIDARG,
				    "cannot create new an_file");
		goto done;
	}

	/* N.B. no copyin of info needed in our implementation */
	err = ani_file_open(path, flags, acd, ifile);

 done:
	if (ani_error_occured(err)) {
		if (ifile)
			NODEOS_DESTROY_FILE(hdomain, ifile);
	} else
		*retval = NODEOS_EXTERN_FILE(hdomain, ifile);

	NODEOS_LEAVE;
	return NODEOS_EXTERN_ERROR(err);
}

/*
 * Include the inlined functions in non-inlined form
 * XXX must be last in the file
 */
#undef NODEOS_FUNC_INLINE
#define NODEOS_FUNC_INLINE

#include <nodeos/impl/an_fs_inline.h>
