/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <assert.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/ani/ani_mutex.h>
#include <nodeos/ani/ani_cond.h>
#include <nodeos/ani/ani_thread.h>

ani_error_t
ani_cond_init(ani_cond_t *newcond)
{
	assert(newcond != 0);
	ani_icond_init(&(newcond->icond));
	ani_ilock_init(&(newcond->ilock));
	return ani_error_clear();
}

ani_error_t
ani_cond_deinit(ani_cond_t *cond)
{
	ani_ilock_deinit(&(cond->ilock));
	ani_icond_deinit(&(cond->icond));
	return ani_error_clear();
}

void
ani_cond_timedwait(ani_cond_t *cond, ani_mutex_t *mutex,
		   an_timespec_t *abstime)
{
	ani_cond_lock(cond);

	/*
	 * Release the mutex and wait.
	 * Note that we are still atomic here since no one can signal the
	 * condition until we release the cond lock.
	 */
	ani_mutex_unlock(mutex);
	ani_thread_condtimedwait(cond, abstime);

	/*
	 * Now unlock the condition before attempting to
	 * reacquire the mutex.  There is no race here.
	 */
	ani_cond_unlock(cond);
	
	ani_mutex_lock(mutex);
}

void
ani_cond_signal(ani_cond_t *cond)
{
	/*
	 * Use the cond's internal lock to synchronize correctly with
	 * waiters.
	 */

	ani_cond_lock(cond);
	ani_icond_signal(&(cond->icond));
	ani_cond_unlock(cond);
}

void
ani_cond_broadcast(ani_cond_t *cond)
{
	ani_cond_lock(cond);
	ani_icond_broadcast(&(cond->icond));
	ani_cond_unlock(cond);
}

/*
 * Include the inlined functions in non-inlined form
 * XXX must be last in the file
 */
#undef NODEOS_FUNC_INLINE
#define NODEOS_FUNC_INLINE

#include <nodeos/ani/ani_cond_inline.h>
