/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <sys/types.h>
#include <sys/stat.h>

#include <nodeos/an_fs.h>
#include <nodeos/ani/ani_fs.h>

static inline void
ani_file_statcvt(an_stat_t *anstat, struct stat *nstat)
{
	anstat->st_atimespec.tv_sec = nstat->st_atime;
	anstat->st_mtimespec.tv_sec = nstat->st_mtime;
	anstat->st_ctimespec.tv_sec = nstat->st_ctime;
	/* no simple, portable way to get nsec resolution. */
	anstat->st_atimespec.tv_nsec = 0;
	anstat->st_mtimespec.tv_nsec = 0;
	anstat->st_ctimespec.tv_nsec = 0;

	anstat->st_acd.st_mode = nstat->st_mode;
	if (S_ISREG(nstat->st_mode))
		anstat->st_type = ANFILE_FILE;
	else if (S_ISDIR(nstat->st_mode))
		anstat->st_type = ANFILE_DIR;
	else
		anstat->st_type = ANFILE_UNKNOWN;
	anstat->st_size = nstat->st_size;
}

ani_error_t 
ani_file_stat(const char *path, an_stat_t *anstat)
{
	int retval;
        struct stat sb;

	retval = stat(path, &sb);
	if (retval < 0)
		return ani_error_set(errno, "native stat call failed");

	ani_file_statcvt(anstat, &sb);
	return ani_error_clear();
}


ani_error_t 
ani_file_fstat(ani_file_t *file, an_stat_t *anstat)
{
        int retval;
        struct stat sb;

	retval = fstat(*file, &sb);
	if (retval < 0)
		return ani_error_set(errno, "native fstat call failed");

	ani_file_statcvt(anstat, &sb);
	return ani_error_clear();
}

/*
 * Include the inlined functions in non-inlined form
 * XXX must be last in the file
 */
#undef NODEOS_FUNC_INLINE
#define NODEOS_FUNC_INLINE

#include <nodeos/ani/ani_fs_inline.h>
