/*
 * Copyright (c) 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Logging functions
 *
 * XXX todo, debug buffering in debug.c should be done here.
 */
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>

#include <nodeos/ani/ani_log.h>

#include "ani_os.h"
#include "ani_debug.h"

void
ani_log_write(int prio, void *buf, int buflen)
{
#ifdef DEBUG
	/* XXX */
	if (prio == ANI_LOG_DEBUG)
		ani_debug_write(buf, buflen);
	else
#endif
	ani_os_log_write(prio, buf, buflen);
}

void
ani_log_printf(int prio, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	ani_log_vprintf(prio, fmt, args);
	va_end(args);
}

void
ani_log_vprintf(int prio, const char *fmt, void *args)
{
	char buf[4096+1];
	int buflen;

	buflen = vsnprintf(buf, sizeof(buf), fmt, args);
	if (buflen > sizeof(buf))
		buflen = sizeof(buf) - 1;
	ani_log_write(prio, buf, buflen);
}
