/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <assert.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/ani/ani_mutex.h>

/*
 * N.B. NodeOS mutexes are not cancelable.
 * This would obviously have to change if the NodeOS doesn't trust the EE.
 */

ani_error_t
ani_mutex_init(ani_mutex_t *newmutex)
{
	assert(newmutex != 0);
	ani_ilock_init(&newmutex->ilock);
	return ani_error_clear();
}

ani_error_t
ani_mutex_deinit(ani_mutex_t *mutex)
{
	ani_ilock_deinit(&mutex->ilock);
	return ani_error_clear();
}

/* ani_mutex_lock is in <nodeos/ani/ani_mutex.h> */
/* ani_mutex_trylock is in <nodeos/ani/ani_mutex.h> */
/* ani_mutex_unlock is in <nodeos/ani/ani_mutex.h> */

/*
 * Include the inlined functions in non-inlined form
 * XXX must be last in the file
 */
#undef NODEOS_FUNC_INLINE
#define NODEOS_FUNC_INLINE

#include <nodeos/ani/ani_mutex_inline.h>
