/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <assert.h>
#include <nodeos/an_resource.h>
#include <nodeos/ani/ani_resource.h>

#include "ani_os.h"

/*
 * Determine the maximum available resources on the node
 */
void
ani_resource_setup(ani_resource_t *noderes)
{
	ani_os_init_resources(noderes);
}

void
ani_resource_initcpu(ani_resource_t *res, an_cpuspec_t *cspec)
{
	/* XXX */
	if (cspec->ttype == AN_TH_IMPLICIT)
		res->threads = cspec->threads.im.maxthreads;
	else
		res->threads = cspec->threads.ex.nthreads;

	/*
	 * XXX if the specified cpu rate is 0, use a default value.
	 * We pick 0.1% of the available cycles (i.e., we can support
	 * 1000 such default domains).
	 */
	if (cspec->rate == 0) {
		extern ani_resource_t ani_moab_noderes;

		assert(ani_moab_noderes.cpurate >= 1000);
		res->cpurate = ani_moab_noderes.cpurate / 1000;
	} else
		res->cpurate = cspec->rate;

	/* XXX hackery */
	res->timeslice = cspec->slice.tv_sec;
	if (res->timeslice != RESOURCE_ALL) {
		assert(cspec->slice.tv_sec < RESOURCE_ALL/1000000);
		res->timeslice = (cspec->slice.tv_sec * 1000000) +
			(cspec->slice.tv_nsec / 1000);
		if (res->timeslice == 0 && cspec->slice.tv_nsec > 0)
			res->timeslice = 1;
	}
}

void
ani_resource_initmem(ani_resource_t *res, an_memspec_t *mspec)
{
	res->memtotal = mspec->nchunks * NODEOS_MEMPOOL_CHUNKSIZE;
}

void
ani_resource_initnet(ani_resource_t *res, an_netspec_t *nspec)
{
	/* XXX */
	res->netrate = 0;
}
