#
# Copyright (c) 1999-2001 University of Utah and the Flux Group.
# All rights reserved.
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

TARGET =	libnodeos.a
TARGET_P =	libnodeos_p.a

SRCS =		$(wildcard $(SRCDIR)/core/*.c) \
		$(wildcard $(SRCDIR)/api_$(NODEOS_API)/*.c)

vpath %.c $(SRCDIR)/core $(SRCDIR)/api_$(NODEOS_API) $(SRCDIR)/os $(SRCDIR)/pq

include $(NODEOS_SRCDIR)/arch.mk


ARCH_SRCS_OSKIT_COMMON = \
		os/oskit_hostname.c os/oskit_sched.c os/oskit_timer.c \
		pq/pqueue.c pq/pq.c os/oskit_log.c os/oskit_net.c \
		os/oskit_chansock.c

ifeq ($(HOST_OS),oskit)
ARCH_SRCS = 	os/oskit.c os/oskit_mem.c os/oskit_dev.c \
		os/oskit_gprof.c os/oskit_fs.c $(ARCH_SRCS_OSKIT_COMMON)
endif

ifeq ($(HOST_OS),oskitunix)
ARCH_SRCS =	os/oskitunix.c os/oskitunix_mem.c os/oskitunix_dev.c \
		os/oskitunix_gprof.c os/oskitunix_fs.c $(ARCH_SRCS_OSKIT_COMMON)
endif

ARCH_SRCS_UNIX_COMMON = \
		os/unix_eth.c os/unix_gprof.c os/unix_fs.c \
		os/unix_timer.c os/unix_hostname.c \
		os/unix_init.c os/unix_sched.c

ifeq ($(HOST_OS),linux)
ARCH_SRCS =	os/linux_log.c os/linux_chaneth.c \
		$(ARCH_SRCS_UNIX_COMMON)
endif

ifeq ($(HOST_OS),solaris)
ARCH_SRCS =	os/solaris_log.c os/solaris_chaneth.c \
		$(ARCH_SRCS_UNIX_COMMON)
endif

ifeq ($(HOST_OS),freebsd)
ARCH_SRCS =	os/freebsd_log.c os/freebsd_chaneth.c \
		$(ARCH_SRCS_UNIX_COMMON)
endif



FILES = 	$(SRCS) $(addprefix $(SRCDIR)/,$(ARCH_SRCS))
FQSRCS =	$(sort $(FILES))

OBJS := $(patsubst %.c,%.o,$(notdir $(FQSRCS)))
POBJS := $(patsubst %.c,%.po,$(notdir $(FQSRCS)))
DEPFILES:= $(patsubst %.c,%.d,$(notdir $(FQSRCS)))

#CFLAGS_WARNINGS=-W -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wnested-externs -Winline
CFLAGS_WARNINGS = -Wwrite-strings -Wstrict-prototypes -Winline -Wbad-function-cast

ifneq ($(HOST_OS),solaris)	# these are too annoying on solaris
CFLAGS_WARNINGS += -Wcast-align -Wshadow
endif

CPPFLAGS = $(OPT_CFLAGS) $(ARCH_CFLAGS) -I$(NODEOS_OBJDIR) -I$(NODEOS_SRCDIR) -I$(NODEOS_SRCDIR)/src -I$(NODEOS_SRCDIR)/src/core
CFLAGS += -Wall $(CFLAGS_WARNINGS) $(CPPFLAGS)

ifeq ($(ENABLE_PROFILING),yes)
all: all_ all_p
install: install_ install_p
else
all: all_
install: install_
endif

all_: $(TARGET)

all_p: $(TARGET) $(TARGET_P)

install_dirs:
	@test -d $(INSTALL_TOPDIR) || mkdir $(INSTALL_TOPDIR)
	@test -d $(INSTALL_LIBDIR) || mkdir $(INSTALL_LIBDIR)

install_: install_dirs $(TARGET)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET)."
	$(INSTALL_DATA) $(TARGET) $(INSTALL_LIBDIR)

install_p: install_dirs $(TARGET_P)
	$(SILENT_MAKE_INFORM) "Installing $(TARGET_P)."
	$(INSTALL_DATA) $(TARGET_P) $(INSTALL_LIBDIR)


$(TARGET): $(OBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

$(TARGET_P): $(POBJS)
	$(SILENT_MAKE_INFORM) "Creating library $@."
	$(AR) cru $@ $^
	$(RANLIB) $@

clean:
	$(SILENT_MAKE_INFORM) "Removing *.o *.po $(TARGET) $(TARGET_P)."
	rm -f .depend *.core *.d *.o *.po $(TARGET) $(TARGET_P)

distclean: clean
	$(SILENT_MAKE_INFORM) "Scrubbing nodeos library directory."
	rm -f GNUmakefile

depend: 
	@echo "WARNING: depend target is deprecated."

mkdebug:
	@echo "ARCH_SRCS = $(ARCH_SRCS)"
	@echo "DEPFILES  = $(DEPFILES)"

-include $(DEPFILES)

.PHONY: depend distclean clean mkdebug
.PHONY: all all_ all_p
.PHONY: install install_dirs install_ install_p 

