/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
#include <stdio.h>

#include <nodeos/config.h>
#include "ani_os.h"

#ifdef CHAN_ETH
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include <net/if.h>
#include <net/bpf.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include <sys/uio.h>

#include <nodeos/impl/an_error.h>
#include <nodeos/ani/ani_nodeos.h>
#include <nodeos/ani/ani_log.h>

static int
bpf_open(void)
{
	int fd;
	int n = 0;
	char dev[sizeof("/dev/bpf000")];
	
	do {
		(void) sprintf(dev, "/dev/bpf%d", n++);
		fd = open(dev, O_RDWR);
	} while (fd < 0 && errno == EBUSY);

	return fd;
}

int
ani_os_open_eth(char *dev)
{
	int fd;
	int buflen;
	struct ifreq ifr;

	if ( (fd = bpf_open()) < 0)
		return -1;

	buflen = NODEOS_CHAN_BUFSIZE;

	if (ioctl(fd, BIOCSBLEN, &buflen) < 0) {
		close(fd);	
		return -1;
	}

	/*
	 * Bind our bpf device to the interface.
	 */
	strncpy(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
	if (ioctl(fd, BIOCSETIF, &ifr) < 0) {
		close(fd);
		return -1;
	}

#if 0
	/*
	 * If we ever need promiscuous mode, this is how to turn it on. 
	 */
	if (ioctl(fd, BIOCPROMISC, &ifr) < 0) {
		close(fd);
		return -1;
	}
#endif	    

	return fd;
}

void
ani_os_close_eth(int fd)
{
	close(fd);
}

int
ani_os_read_eth(int fd, char *buf, int bufsize, int timeo)
{
	struct bpf_hdr *hdr;
	static char ourbuf[NODEOS_CHAN_BUFSIZE];
	static int ofs = 0;
	static int count = 0;
	int len = 0;
	fd_set fdset;
	struct timeval tv;

	if (ofs == 0) {
		if (timeo > 0) {
			FD_ZERO(&fdset);
			FD_SET(fd, &fdset);
			tv.tv_sec = timeo / 1000;
			tv.tv_usec = (timeo % 1000) * 1000;
			count = select(fd+1, &fdset, 0, 0, &tv);
			if (count <= 0)
				return count;
		}
		while ((count = read(fd, &ourbuf[0], bufsize)) < 0) {
			if (errno == EWOULDBLOCK) 
				return 0;
			if (errno != EINTR)
				return -1;
		}
	}
		
	if (count != 0) {
		hdr = (struct bpf_hdr *)(&ourbuf[0] + BPF_WORDALIGN(ofs));
		len = hdr->bh_caplen;

		if (hdr->bh_caplen != hdr->bh_datalen)
			ani_log_printf(ANI_LOG_WARN,
				       "%s: truncated packet!\n", __FUNCTION__);
		
		memcpy(buf, &ourbuf[0] + ofs + hdr->bh_hdrlen, len);

		ofs += hdr->bh_hdrlen + hdr->bh_caplen;
		ofs = BPF_WORDALIGN(ofs);

		if (ofs >= count) 
			ofs = count = 0;
	}

	return len;
}

/*
 * XXX assumes socket is connected
 * XXX pass timeout value?
 */
int
ani_os_write_eth(int fd, char *buf, int bufsize)
{
	return (int)write(fd, buf, bufsize);
}
#endif /* CHAN_ETH */
