/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#include <stdio.h>
#include <signal.h>
#include <stdarg.h>

#include <nodeos/ani/ani_log.h>
#include "ani_os.h"

static void sigcontext_dump_stack_trace(struct sigcontext *scp);

/*
 * Write a buffer to a logfile.
 *
 * Note: this function may be called very early in the initialization
 * or very late in shutdown of Moab and hence must work independent of the
 * state of the NodeOS.
 */
void
ani_os_log_write(int prio, void *buf, int buflen)
{
	FILE *fd = stdout;

	if (prio == ANI_LOG_DEBUG)
		fd = stderr;
	fwrite(buf, 1, buflen, fd);
}

/*
 * Emergency shutdown function
 */
void
ani_os_panic(const char *fmt, ...)
{
	va_list vl;

	va_start(vl, fmt);
	vprintf(fmt, vl);
	va_end(vl);

	printf("\n");

	_exit(1);
}

/*
 * Linux signal-state dump
 */
void
ani_os_dump_state(void *state)
{
	struct sigcontext *scp = state;
	static int indump;

	if (indump++)
		return;

	printf("EAX %08x EBX %08x ECX %08x EDX %08x\n",
	       scp->sc_eax, scp->sc_ebx, scp->sc_ecx, scp->sc_edx);
	printf("ESI %08x EDI %08x EBP %08x ESP %08x\n",
	       scp->sc_esi, scp->sc_edi, scp->sc_ebp, scp->sc_esp);
	printf("EIP %08x EFLAGS %08x\n", scp->sc_eip, scp->sc_efl);
	printf("CS %04x SS %04x DS %04x ES %04x\n",
	       scp->sc_cs & 0xffff, scp->sc_ss & 0xffff,
	       scp->sc_ds & 0xffff, scp->sc_es & 0xffff);
	sigcontext_dump_stack_trace(scp);
	indump = 0;
}

static void
sigcontext_dump_stack_trace(struct sigcontext *scp)
{
	unsigned int *fp, i;

	fp = (unsigned int *)scp->sc_ebp;

	printf("Backtrace:");
	for (i = 0; i < 16; i++) {
		if ((i % 8) == 0)
			printf("\n");

		fp = (int *)(*fp);
		if (!(*(fp + 1) && *fp))
			break;
		printf(" %08x", *(fp + 1));
	}
	printf("\n");
}
