/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Hodge-podge of stuff for OSKit
 */
#include <oskit/clientos.h>
#include <oskit/startup.h>
#include <oskit/threads/pthread.h>
#include <nodeos/config.h>

#include "ani_os.h"

#include <nodeos/an_thread.h>
#include <nodeos/impl/an_error.h>
#include <nodeos/ani/ani_thread.h>

#define DOCTRLC

#ifdef DOCTRLC
#define CHKPERSEC	2	/* ^C checks per second */

static
void ctrlc_me(void *oskitthingee, void *arg)
{
	int com_cons_trygetchar(int port);
	void com_cons_putchar(int port, int ch);

#if 0
	com_cons_putchar(1, '.');
#endif
	if (com_cons_trygetchar(1) == 3)
		ani_os_panic("^C\nmoab aborted\n");
}
#endif

/*
 * Get the basic OSKit ``stuff'' up and running.
 */
void
ani_os_init_basic(void)
{
	oskit_clientos_init_pthreads();
	start_clock();
	start_pthreads();

#ifdef DOCTRLC
	/* fire off a debug timer */
	{
		void *mytimer;
		unsigned long ticksize;

		mytimer = ani_os_timer_init(ctrlc_me, 0, &ticksize);
		assert(mytimer != 0);
		assert(ticksize <= 1000000/CHKPERSEC);
		ani_os_timer_start(mytimer, 1000000/CHKPERSEC);
	}
#endif
}

void
ani_os_init_resources(ani_resource_t *res)
{
	/*
	 * XXX Fer now hack
	 */
	res->threads = 1000;		/* units: # of threads */
	res->timeslice = 10000;		/* units: microseconds */
	res->cpurate = 1*1000*1000;	/* units: cycles per second */
	res->memtotal = 128*1024*1024;	/* units: bytes */
	res->netrate = 100*1000*1000;	/* units: bits per second */

	res->stacksize = 128 * 1024;	/* XXX doesn't belong */
}

void
ani_os_time_current(an_timespec_t *curtime)
{
	struct timeval tv;
	int err;

	err = gettimeofday(&tv, 0);
	assert(err == 0);
	curtime->tv_sec = tv.tv_sec;
	curtime->tv_nsec = tv.tv_usec * 1000;
}
