/*
 * Copyright (c) 2001, 2002 University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * OSKit-specific setup/teardown of BSD networking stack
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <oskit/threads/pthread.h>
#include <oskit/startup.h>
#ifdef UTAHTESTBED
#include <oskit/tmcp.h>
#endif

#include <nodeos/an_info.h>
#include <nodeos/ani/ani_log.h>

#include "ani_os.h"
#include "ani_debug.h"

#include <nodeos/config.h>

#ifdef CHAN_SOCKET
int
ani_os_init_chansock(void)
{
	oskit_socket_factory_t *fsc;
	int i, err, ndev;
	char *addr, *mask;
	char *hname, *gwname, *dname, **nsnames;
	an_infoipv4_t *ipv4info;

	err = start_conf_network_init(start_osenv(), &fsc);
	if (err != 0)
		return err;
	pthread_init_socketfactory(fsc);

	ipv4info = ani_os_getipv4info(&ndev);

	/*
	 * Find all interfaces and configure them
	 */
	for (i = 0; i < ndev; i++) {
		struct in_addr tmp;

		if (ipv4info[i].addr == 0 && ipv4info[i].mask == 0)
			continue;

		tmp.s_addr = ipv4info[i].addr;
		addr = strdup(inet_ntoa(tmp));
		tmp.s_addr = ipv4info[i].mask;
		mask = strdup(inet_ntoa(tmp));
		err = start_conf_network_eifstart(ipv4info[i].ifid, addr, mask);
		if (err != 0) {
			ani_log_printf(ANI_LOG_WARN, "WARNING: "
				       "if%d: ifconfig %s/%s failed (err=%x)\n",
				       ipv4info[i].ifid, addr, mask, err);
			continue;
		}
		free(addr);
		free(mask);
	}

	/*
	 * Setup our host configuration
	 */
#ifdef UTAHTESTBED
	err = start_tmcp_getinfo_host(&hname, &dname, &gwname, &nsnames);
#else
	err = start_getinfo_network_host(&hname, &dname, &gwname, &nsnames, 1);
#endif
	if (err == 0)
		err = start_conf_network_host(hname, gwname, dname, nsnames);
	else
		ani_log_printf(ANI_LOG_WARN, "WARNING: "
			       "could not obtain network host information\n");

	/*
	 * Summarize
	 */
	DBG(DBG_DEVICE_INIT, {
		ani_dprintf("IP configuration");
		if (hname != 0) {
			ani_dprintf(" for %s", hname);
			if (dname != 0)
				ani_dprintf(".%s", dname);
		}
		ani_dprintf(":\n");
		if (gwname != 0)
			ani_dprintf("  default gateway=%s\n", gwname);
		if (nsnames != 0 && nsnames[0] != 0)
			ani_dprintf("  primary nameserver=%s\n", nsnames[0]);
	})

	/* XXX do we need this info around? */
	if (hname != 0)
		free(hname);
	if (gwname != 0)
		free(gwname);
	if (dname != 0)
		free(dname);
	if (nsnames != 0) {
		char **ns;

		for (ns = nsnames; *ns != 0; ns++)
			free(*ns);
		free(nsnames);
	}

	return 0;
}

void
ani_os_deinit_chansock(void)
{
	int i, ndev;
	an_infoipv4_t *ipv4info;

	ipv4info = ani_os_getipv4info(&ndev);
	for (i = 0; i < ndev; i++) {
		if (start_conf_network_eifstop(ipv4info[i].ifid) != 0)
			ani_log_printf(ANI_LOG_WARN, "WARNING: "
				       "could not shutdown eth%d\n",
				       ipv4info[i].ifid);
	}
}
#endif
