/*
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

#ifdef GPROF

#include <stdio.h>
#include <oskit/startup.h>

#include "ani_os.h"

static const char *_gprof_args[] = {"gprof", "nodeos_p", 0};
static int _gprof_hz = 1024;	/* range: 128 to 8192 */

static void oskit_gprof_dump(void *arg);
void gprof(int argc, const char **argv);

/*
 * Equiv of startup/start_gprof.
 * We set the sample rate and use our own cleanup so we can call gprof
 * with arguments.
 */
void
ani_os_gprof_init(void)
{
	extern int enable_gprof, gprof_hz;
	void oskit_dump_gprof(void *);
	void _mcleanup(void *);

	if (!enable_gprof) {
		ani_log_printf(ANI_LOG_WARN,
			       "%s: GPROF is not enabled in kern library!",
			       __FUNCTION__);
		return;
	}
	
#if 0
	/* XXX ani_os_fs_init should be done first and take care of this */
	start_fs_bmod_pthreads();
#endif

	gprof_hz = _gprof_hz;
	startup_atexit(oskit_gprof_dump, NULL);
	startup_atexit(_mcleanup, NULL);
}

void
ani_os_gprof_start(const char *kernelname)
{
	extern unsigned long _start, etext;
	void monstartup(unsigned long lowpc, unsigned long highpc);

	if (kernelname)
		_gprof_args[1] = kernelname;

	ani_log_printf(ANI_LOG_INFO,
		       "Starting profile of `%s' ([%p-%p]), %d samples/sec\n",
		       _gprof_args[1], &_start, &etext, _gprof_hz);
	monstartup((unsigned long)&_start, (unsigned long)&etext);
}

void
ani_os_gprof_stop(void)
{
	void moncontrol(int mode);

	moncontrol(0);
	ani_log_printf(ANI_LOG_INFO, "Profile done\n");
}

static void
oskit_gprof_dump(void *arg)
{
	gprof(2, _gprof_args);
}

/*
 * Override this to prevent it from happening out of multiboot_main.
 */
void
base_gprof_init(void)
{
}

#endif

