/*
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

/*
 * Device related initialization for OSKit/unix
 * XXX lots of overlap with raw OSKit version
 */
#include <stdio.h>

#include <oskit/dev/dev.h>
#include <oskit/dev/osenv.h>
#include <oskit/dev/blk.h>
#include <oskit/dev/ethernet.h>
#include <oskit/dev/bus.h>
#include <oskit/dev/linux.h>

#include <oskit/clientos.h>
#include <oskit/startup.h>
#include <oskit/threads/pthread.h>
#include <nodeos/config.h>

#include "ani_debug.h"

#if defined(CHAN_ETH) || defined(CHAN_SOCKET)
#define HAVE_NET
#endif

#if defined(HAVE_NET) || defined(HAVE_DISK)
#define HAVE_DEVICES
#endif

/*
 * Initialize and probe device drivers.
 */
void
ani_os_init_devices(void)
{
#ifdef HAVE_DEVICES
	osenv_process_lock();
	start_devices();
	osenv_process_unlock();
#endif
}

#ifdef HAVE_DEVICES
/*
 * This is our own version of start devices so we can be more selective.
 * Comment this out, if you want all the enet and disk devices.
 */
void
start_devices(void)
{
	static void init_net(void);
	static void init_disk(void);
	oskit_osenv_t *osenv;
	static int run;
	
	if (run++)
		return;

	DBG(DBG_DEVICE_INIT, ani_dprintf("Initializing device drivers...\n"));
	osenv = start_osenv();
	oskit_dev_init(osenv);
	oskit_linux_init_osenv(osenv);

#ifdef HAVE_NET
	init_net();
#endif
#ifdef HAVE_DISK
	init_disk();
#endif

	DBG(DBG_DEVICE_INIT, oskit_dump_drivers());

	DBG(DBG_DEVICE_INIT, ani_dprintf("Probing devices...\n"));
	oskit_dev_probe();

	DBG(DBG_DEVICE_INIT, oskit_dump_devices());
}

#ifdef HAVE_NET
/*
 * Equiv of oskit_linux_init_ethernet
 *
 * Insert individual drivers you want or call oskit_linux_init_ethernet
 * for all of them.
 */
static void
init_net(void)
{
	oskit_linux_init_net();
}

/*
 * XXX override startup library.
 *
 * This routine would be pulled in by startup_network_router and the
 * default version would pull in all enet drivers.
 */
void
start_net_devices(void)
{
}
#endif

#ifdef HAVE_DISK
/*
 * Equiv of oskit_linux_init_blk
 *
 * Insert individual drivers you want or call oskit_linux_init_blk
 * for all of them.
 */
static void
init_disk(void)
{
}
#endif
#endif
